#!/usr/bin/python

#
# Related BZ:
#       1255552
#


import sys

from libsan.host.cmdline import run
from libsan.host.dmpd import (
    cache_check,
    cache_dump,
    cache_restore,
    thin_check,
    thin_dump,
    thin_restore,
)
from libsan.host.linux import install_package
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.lvm import (
    lv_activate,
    lv_convert,
    lv_create,
    lv_deactivate,
    vg_create,
    vg_remove,
)

import stqe.host.tc
from stqe.host.atomic_run import atomic_run

TC = None


def _mount_thin_lv(vg_name, lv_name):
    run("mkdir -p /mnt/%s" % lv_name, verbose=True)
    run(f"mkfs.ext4 /dev/{vg_name}/{lv_name}", verbose=True)
    run(f"mount /dev/{vg_name}/{lv_name} /mnt/{lv_name}/", verbose=True)


def _unmount_thin_lv(lv_name):
    run("umount -f /mnt/%s/" % lv_name, verbose=True)
    run("rm -rf /mnt/%s" % lv_name, verbose=True)


def create_lvs(args, iteration, errors):
    if iteration == 1:
        atomic_run(
            "Creating thin LV.",
            vg_name=args["group"] + "/" + args["pool"],
            lv_name=args["vol"],
            options=["-T", "-V 10"],
            command=lv_create,
            errors=errors,
        )

        atomic_run(
            "Deactivating thin LV.",
            lv_name=args["vol"],
            vg_name=args["group"],
            command=lv_deactivate,
            errors=errors,
        )

    elif iteration == 2:
        atomic_run(
            "Creating thin LV.",
            vg_name=args["group"] + "/" + args["pool"],
            lv_name=args["vol"],
            options=["-T", "-V 10"],
            command=lv_create,
            errors=errors,
        )

        atomic_run(
            "Creating filesystem on LV.",
            cmd="mkfs.ext4 /dev/{}/{}".format(args["group"], args["vol"]),
            command=run,
            errors=errors,
        )

        atomic_run(
            "Deactivating thin LV.",
            lv_name=args["vol"],
            vg_name=args["group"],
            command=lv_deactivate,
            errors=errors,
        )

    elif iteration == 3:
        for i in range(2):
            atomic_run(
                "Creating thin LV No. %s." % i,
                vg_name=args["group"] + "/" + args["pool"],
                lv_name=args["vol"] + str(i),
                options=["-T", "-V 10"],
                command=lv_create,
                errors=errors,
            )

            atomic_run(
                "Deactivating thin LV No. %s." % i,
                lv_name=args["vol"] + str(i),
                vg_name=args["group"],
                command=lv_deactivate,
                errors=errors,
            )

    elif iteration == 4:
        for i in range(2):
            atomic_run(
                "Creating thin LV No. %s." % i,
                vg_name=args["group"] + "/" + args["pool"],
                lv_name=args["vol"] + str(i),
                options=["-T", "-V 10"],
                command=lv_create,
                errors=errors,
            )
            if i == 1:
                atomic_run(
                    "Creating filesystem on LV No. %s." % i,
                    cmd="mkfs.ext4 /dev/{}/{}".format(args["group"], args["vol"] + str(i)),
                    command=run,
                    errors=errors,
                )

            atomic_run(
                "Deactivating thin LV No. %s." % i,
                lv_name=args["vol"] + str(i),
                vg_name=args["group"],
                command=lv_deactivate,
                errors=errors,
            )

    elif iteration == 5:
        for i in range(2):
            atomic_run(
                "Creating thin LV No. %s." % i,
                vg_name=args["group"] + "/" + args["pool"],
                lv_name=args["vol"] + str(i),
                options=["-T", "-V 10"],
                command=lv_create,
                errors=errors,
            )
            if i == 2:
                atomic_run(
                    "Creating filesystem on LV No. %s." % i,
                    cmd="mkfs.ext4 /dev/{}/{}".format(args["group"], args["vol"] + str(i)),
                    command=run,
                    errors=errors,
                )

            atomic_run(
                "Deactivating thin LV No. %s." % i,
                lv_name=args["vol"] + str(i),
                vg_name=args["group"],
                command=lv_deactivate,
                errors=errors,
            )

    elif iteration == 6:
        for i in range(2):
            atomic_run(
                "Creating thin LV No. %s." % i,
                vg_name=args["group"] + "/" + args["pool"],
                lv_name=args["vol"] + str(i),
                options=["-T", "-V 10"],
                command=lv_create,
                errors=errors,
            )

            atomic_run(
                "Creating filesystem on LV No. %s." % i,
                cmd="mkfs.ext4 /dev/{}/{}".format(args["group"], args["vol"] + str(i)),
                command=run,
                errors=errors,
            )

            atomic_run(
                "Deactivating thin LV No. %s." % i,
                lv_name=args["vol"] + str(i),
                vg_name=args["group"],
                command=lv_deactivate,
                errors=errors,
            )


def thin_init(args, iteration):
    print("INFO: Initializing test case, iteration No. %s" % iteration)
    errors = []

    atomic_run(
        "Creating loopdev",
        name=args["loop"],
        size=args["loop_size"],
        command=create_loopdev,
        errors=errors,
    )

    atomic_run(
        "Creating VG",
        vg_name=args["group"],
        pv_name="/dev/" + args["loop"],
        command=vg_create,
        errors=errors,
    )

    atomic_run(
        "Creating thin pool",
        vg_name=args["group"],
        lv_name=args["pool"],
        options=["-T", "-L 100"],
        command=lv_create,
        errors=errors,
    )

    # Create LVs each time a bit different to cover all the conditions
    create_lvs(args, iteration, errors)

    atomic_run(
        "Deactivating pool",
        lv_name=args["pool"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Creating swap LV",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=["-L 100"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Deactivating swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Swapping metadata",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=[
            "-y",
            "--thinpool " + args["group"] + "/" + args["pool"],
            "--poolmetadata ",
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Activating swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_activate,
        errors=errors,
    )

    if len(errors) == 0:
        TC.tpass("Initialization No. %s passed" % iteration)
    else:
        TC.tfail(
            "Initialization No. %s failed with following errors: \n\t'" % iteration
            + "\n\t ".join([str(i) for i in errors]),
        )
        return 1
    return 0


def cache_init(args):
    print("INFO: Initializing test case.")
    errors = []

    atomic_run(
        "Creating loopdev 1 - 'fast' device",
        name=args["loop"],
        size=args["loop_size"],
        command=create_loopdev,
        errors=errors,
    )

    atomic_run(
        "Creating loopdev 2 - 'slow' device",
        name=args["loop2"],
        size=args["loop2_size"],
        command=create_loopdev,
        errors=errors,
    )

    atomic_run(
        "Creating VG",
        vg_name=args["group"],
        pv_name="/dev/" + args["loop"] + " /dev/" + args["loop2"],
        command=vg_create,
        errors=errors,
    )

    atomic_run(
        "Creating cache metadata volume",
        vg_name=args["group"] + " /dev/" + args["loop"],
        lv_name=args["meta"],
        options=["-L 12"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Creating origin volume",
        vg_name=args["group"] + " /dev/" + args["loop2"],
        lv_name=args["origin"],
        options=["-L 2G"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Creating cache data volume",
        vg_name=args["group"] + " /dev/" + args["loop"],
        lv_name=args["data"],
        options=["-L 1G"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Creating cache pool",
        vg_name=args["group"],
        lv_name=args["data"],
        options=[
            "-y --type cache-pool",
            "--cachemode writeback",
            "--poolmetadata {}/{}".format(args["group"], args["meta"]),
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Creating cache logical volume",
        vg_name=args["group"],
        lv_name=args["origin"],
        options=[
            "-y",
            "--type cache",
            "--cachepool {}/{}".format(args["group"], args["data"]),
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Splitting cache logical volume",
        vg_name=args["group"],
        lv_name=args["origin"],
        options=["-y", "--splitcache"],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Creating swap LV",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=["-L 100"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Swapping metadata",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=[
            "-y",
            "--cachepool " + args["group"] + "/" + args["data"],
            "--poolmetadata ",
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Activating swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_activate,
        errors=errors,
    )

    if len(errors) == 0:
        TC.tpass("Initialization passed")
    else:
        TC.tfail("Initialization failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def thin_clean(args):
    print("INFO: Cleaning up")
    errors = []

    # restoring metadata device after corrupting it
    atomic_run(
        "Repairing metadata device",
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        quiet=True,
        command=thin_restore,
        errors=errors,
    )

    # thinpool got activated after checking its metadata to get bad checksum
    atomic_run(
        "Deactivating pool",
        lv_name=args["pool"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Deactivating swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Swapping back metadata",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=[
            "-y",
            "--thinpool " + args["group"] + "/" + args["pool"],
            "--poolmetadata ",
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Removing VG",
        vg_name=args["group"],
        force=True,
        command=vg_remove,
        errors=errors,
    )

    atomic_run("Deleting loopdev", name=args["loop"], command=delete_loopdev, errors=errors)

    atomic_run(
        "Deleting metadata file",
        cmd="rm -f /var/tmp/metadata",
        command=run,
        errors=errors,
    )

    if len(errors) == 0:
        TC.tpass("Cleanup passed")
    else:
        TC.tfail("Cleanup failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors]))
        print(errors)
        return 1
    return 0


def cache_clean(args):
    print("INFO: Cleaning up")
    errors = []

    atomic_run(
        "Removing VG",
        vg_name=args["group"],
        force=True,
        command=vg_remove,
        errors=errors,
    )

    atomic_run(
        "Deleting loopdev loop1",
        name=args["loop"],
        command=delete_loopdev,
        errors=errors,
    )

    atomic_run(
        "Deleting loopdev loop2",
        name=args["loop2"],
        command=delete_loopdev,
        errors=errors,
    )

    atomic_run(
        "Deleting metadata file",
        cmd="rm -f /var/tmp/metadata",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Deleting repair metadata file",
        cmd="rm -f /var/tmp/metadata_repair",
        command=run,
        errors=errors,
    )

    if len(errors) == 0:
        TC.tpass("Cleanup passed")
    else:
        TC.tfail("Cleanup failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors]))
        print(errors)
        return 1
    return 0


def install_packages(packages):
    for package in packages:
        install_package(package)


def thin(args, iteration):
    print("\n#######################################\n")
    print("INFO: Testing restoring corruption with thin_restore, iteration No. %s" % iteration)

    errors = []

    atomic_run(
        "Dumping metadata to file",
        formatting="xml",
        source_vg=args["group"],
        source_lv=args["swap"],
        output="/var/tmp/metadata",
        command=thin_dump,
        errors=errors,
    )

    atomic_run(
        "Checking metadata",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=thin_check,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata from nonexistent file",
        False,
        cmd="thin_restore -i /var/tmp/wrong.wrong -o /dev/mapper/{}-{}".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    # should pass without error on the last iteration
    if iteration != 6:
        atomic_run(
            "Checking metadata",
            pkg=["device-mapper-persistent-data", "0.7.3", "1.el7"],
            source_vg=args["group"],
            source_lv=args["swap"],
            command=thin_check,
            errors=errors,
        )
    else:
        atomic_run(
            "Checking metadata",
            source_vg=args["group"],
            source_lv=args["swap"],
            command=thin_check,
            errors=errors,
        )

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing restoring corruption with thin_restore passed, iteration No. %s" % iteration)
    else:
        TC.tfail(
            "Testing restoring corruption with thin_restore failed with following errors: \n\t'"
            + "\n\t ".join([str(i) for i in errors])
            + "\nIteration No. %s" % iteration,
        )
        return 1
    return 0


def cache(args):
    print("\n#######################################\n")
    print("INFO: Testing restoring corruption with cache_restore.")

    errors = []

    atomic_run(
        "Dumping metadata to file",
        source_vg=args["group"],
        source_lv=args["swap"],
        output="/var/tmp/metadata",
        command=cache_dump,
        errors=errors,
    )

    atomic_run(
        "Checking metadata",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=cache_check,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with options",
        False,
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        omit_clean_shutdown=True,
        command=cache_restore,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata from nonexistent file",
        False,
        cmd="cache_restore -i /var/tmp/wrong.wrong -o /dev/mapper/{}-{}".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    # The fail here is caused by using "omit-clean-shotdown" before
    atomic_run(
        "Checking metadata",
        pkg=["device-mapper-persistent-data", "0.7.3", "1.el7"],
        source_vg=args["group"],
        source_lv=args["swap"],
        command=cache_check,
        errors=errors,
    )

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing restoring corruption with thin_restore passed")
    else:
        TC.tfail(
            "Testing restoring corruption with thin_restore failed with following errors: \n\t'"
            + "\n\t ".join([str(i) for i in errors]),
        )
        return 1
    return 0


def main():
    # Initialize Test Case
    global TC
    TC = stqe.host.tc.TestClass()

    args = {
        "loop": "loop1",
        "loop_size": 2048,
        "loop2": "loop2",
        "loop2_size": 4128,
        "group": "vgtest",
        "pool": "thinpool",
        "vol": "thinvol",
        "number of vols": 10,
        "origin": "origin",
        "data": "cache_data",
        "meta": "cache_meta",
        "swap": "swapvol",
    }

    # Initialization
    packages = ["device-mapper-persistent-data"]
    install_packages(packages)

    # thin_restore testing
    for i in range(1, 7):
        thin_init(args, i)
        thin(args, i)
        thin_clean(args)

    # cache_restore testing
    cache_init(args)
    cache(args)
    cache_clean(args)

    if not TC.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_create_success():
    errors = []

    vol_name = read_var("LSM_VOL_NAME")
    vol_size = read_var("LSM_VOL_SIZE")
    pool_id = read_var("LSM_POOL_ID_VOLUME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        for provisioning in [None, "DEFAULT", "THIN", "FULL"]:
            provisioning = {"provisioning": provisioning} if provisioning is not None else {}
            _, data = atomic_run(
                "Creating volume {} with protocol {}.".format(vol_name, config["protocol"]),
                command=lsm.volume_create,
                name=vol_name,
                size=vol_size,
                pool=pool_id,
                return_output=True,
                errors=errors,
                **provisioning,
            )
            vol_id = [line.split()[0].strip() for line in data.splitlines() if vol_name in line][0]
            atomic_run(
                "Removing volume {} with protocol {}.".format(vol_name, config["protocol"]),
                command=lsm.volume_delete,
                vol=vol_id,
                force=True,
                errors=errors,
            )
    return errors


def volume_create_fail():
    errors = []

    vol_name = read_var("LSM_VOL_NAME")
    vol_size = read_var("LSM_VOL_SIZE")
    pool_id = read_var("LSM_POOL_ID_VOLUME")
    pool_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail creating volume without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_create,
            },
            {
                "message": "Trying to fail creating volume without name with protocol %s" % config["protocol"],
                "command": lsm.volume_create,
                "size": vol_size,
                "pool": pool_id,
            },
            {
                "message": "Trying to fail creating volume without size with protocol %s" % config["protocol"],
                "command": lsm.volume_create,
                "name": vol_name,
                "pool": pool_id,
            },
            {
                "message": "Trying to fail creating volume without pool with protocol %s" % config["protocol"],
                "command": lsm.volume_create,
                "size": vol_size,
                "name": vol_name,
            },
            {
                "message": "Trying to fail creating volume with wrong size '-1' with protocol %s" % config["protocol"],
                "command": lsm.volume_create,
                "size": -1,
                "name": vol_name,
                "pool": pool_id,
            },
            {
                "message": "Trying to fail creating volume with wrong size '0' with protocol %s" % config["protocol"],
                "command": lsm.volume_create,
                "size": 0,
                "name": vol_name,
                "pool": pool_id,
            },
            {
                "message": "Trying to fail creating volume with wrong size 'wrong' with protocol %s"
                % config["protocol"],
                "command": lsm.volume_create,
                "size": "wrong",
                "name": vol_name,
                "pool": pool_id,
            },
            {
                "message": "Trying to fail creating volume with pool 'NONEXISTENT' with protocol %s"
                % config["protocol"],
                "command": lsm.volume_create,
                "size": vol_size,
                "name": vol_name,
                "pool": "NONEXISTENT",
            },
            {
                "message": "Trying to fail creating volume with pool name instead of ID with protocol %s"
                % config["protocol"],
                "command": lsm.volume_create,
                "size": vol_size,
                "name": vol_name,
                "pool": pool_name,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def volume_create_fail_name_conflict():
    errors = []

    vol_name = read_var("LSM_VOL_NAME")
    vol_size = read_var("LSM_VOL_SIZE")
    pool_id = read_var("LSM_POOL_ID_VOLUME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Creating volume {} again to hit NAME CONFLICT with protocol {}.".format(vol_name, config["protocol"]),
            command=lsm.volume_create,
            name=vol_name,
            size=vol_size,
            pool=pool_id,
            expected_ret=4,
            errors=errors,
        )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = volume_create_success()
    if int(environ["fmf_tier"]) == 2:
        errs = volume_create_fail()
        errs += volume_create_fail_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, get_func_from_string, read_env
from stqe.host.persistent_vars import read_var, write_var
from stqe.host.stratis import get_stratis_args


def write_data(args, errors):
    name = read_env("fmf_name")
    if "fs_create" in name:
        atomic_run(
            "Writing var STRATIS_FS",
            command=write_var,
            var={"STRATIS_FS": args["fs_name"]},
            errors=errors,
        )

    if "snapshot_create" in name:
        atomic_run(
            "Writing var STRATIS_SNAPSHOT",
            command=write_var,
            var={"STRATIS_SNAPSHOT": args["snapshot_name"]},
            errors=errors,
        )

    if "mkdir" in name:
        atomic_run(
            "Writing var FS_DIR",
            command=write_var,
            var={"FS_DIR": args["cmd"].split()[-1]},
            errors=errors,
        )
    return


def setup():
    errors = []

    stratis = Stratis(disable_check=True)
    args = get_stratis_args(stratis_object=stratis)
    print(args)

    # some extra args for some setup
    args.update(get_env_args(["id", "cmd", "path"], read_var("FILE_NAMES")))
    command = args.pop("command")

    args["command"] = get_func_from_string(stratis, command)
    atomic_run(errors=errors, **args)
    print("the second args %s" % args)
    write_data(args, errors)

    return errors


if __name__ == "__main__":
    errs = setup()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_var


def cleanup_raid():
    raid = read_var("VDO_DEVICE")
    print("INFO: Cleaning up raid %s" % raid)
    errors = []

    atomic_run(
        "Stopping raid %s" % raid,
        cmd="mdadm --stop %s" % raid,
        command=run,
        errors=errors,
    )

    disks = read_var("RAID_DISKS")
    atomic_run(
        "Zeroing superblock on disks: %s" % disks,
        cmd="mdadm --zero-superblock %s" % disks,
        command=run,
        errors=errors,
    )

    atomic_run("Cleaning var VDO_DEVICE", command=clean_var, var="VDO_DEVICE", errors=errors)

    atomic_run("Cleaning var RAID_DISKS", command=clean_var, var="RAID_DISKS", errors=errors)

    return errors


if __name__ == "__main__":
    errs = cleanup_raid()
    exit(parse_ret(errs))



#!/usr/bin/python

from libsan.host.cmdline import run
from libsan.host.scsi import get_free_disks

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var


def setup_crypt(crypt_name=None):
    errors = []
    passwd = "redhatredhat"
    if not crypt_name:
        crypt_name = "test_crypt"
    disks = atomic_run(message="Getting free disks for crypt", command=get_free_disks, errors=errors)
    if disks is None:
        exit(1)
    disks = disks.keys()
    _, lsblk = atomic_run(
        message="Listing block devices to check they are really free.",
        command=run,
        cmd="lsblk",
        return_output=True,
        errors=errors,
    )
    # remove disks that are in use (find only partitioned ones)
    values = [x.strip() for line in lsblk.splitlines() for x in line.split()]
    for disk in [disk for value in values for disk in disks if disk != value and disk in value]:
        disks.remove(disk)

    disk_paths = ["/dev/" + j for j in disks]
    print("Found these free disks: %s" % " ".join(disk_paths))
    for disk in disk_paths:
        atomic_run(
            "Zeroing superblock of disk %s." % disk,
            command=run,
            cmd="dd if=/dev/zero of=%s bs=4k count=2" % disk,
            errors=errors,
        )
    if not disk_paths:
        print("Could not find any disk")
        exit(1)
    test_disk = disk_paths[0]
    atomic_run(
        "Format luks with device %s" % test_disk,
        cmd=f"echo {passwd} |cryptsetup luksFormat -q {test_disk}",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Open luks with device %s " % test_disk,
        cmd=f"echo {passwd} |cryptsetup open -q {test_disk}  {crypt_name}",
        command=run,
        errors=errors,
    )

    run(cmd="cryptsetup luksDump %s" % test_disk)

    atomic_run(
        "Writing var STRATIS_AVAILABLE_DEVICES",
        command=write_var,
        var={"STRATIS_AVAILABLE_DEVICES": "/dev/mapper/%s" % crypt_name},
        errors=errors,
    )

    atomic_run(
        "Writing var STRATIS_DEVICE",
        command=write_var,
        var={"STRATIS_DEVICE": "/dev/mapper/%s" % crypt_name},
        errors=errors,
    )

    atomic_run("Writing var CRYPT_DISKS", command=write_var, var={"CRYPT_DISKS": test_disk}, errors=errors)

    atomic_run(message="Listing block devices.", command=run, cmd="lsblk", errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_crypt(crypt_name="test_crypt")
    exit(parse_ret(errs))



# Copyright (C) 2022 Red Hat, Inc.
# This file is part of libsan.
#
# libsan is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libsan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libsan.  If not, see <http://www.gnu.org/licenses/>.


import unittest
from unittest.mock import call, patch

import stqe.host.logchecker as log_checker
from libsan.host.cmdline import run

# found segfault
segfault_msg = (
    " segfault at 10 ip 00007f9bebcca90d sp 00007fffb62705f0 error 4 in libQtWebKit.so.4.5.2[7f9beb83a000+f6f000]"
)
calltrace_msg = " Call Trace: "


def _run_dmesg_segfault(cmd, **kwargs):
    if cmd.startswith("dmesg | grep"):
        cmd = cmd.replace("dmesg", "echo '%s'" % segfault_msg)
        return run(cmd, verbose=False, **kwargs)

    return (0, "")


def _run_dmesg_calltrace(cmd, **kwargs):
    if cmd.startswith("dmesg | grep"):
        cmd = cmd.replace("dmesg", "echo '%s'" % calltrace_msg)
        return run(cmd, verbose=False, **kwargs)

    return (0, "")


class Testlogchecker(unittest.TestCase):
    @patch("stqe.host.logchecker.run")
    def test_kernel_check(self, run_func):
        run_func.return_value = (0, 0)
        assert log_checker.kernel_check() is True
        # already handled taint
        run_func.return_value = (0, 1)
        assert log_checker.kernel_check() is True

    @patch("stqe.host.logchecker.run")
    def test_dmesg_check(self, run_func):
        run_func.return_value = (0, "")
        assert log_checker.dmesg_check() is True
        run_func.reset_mock()

        run_func.side_effect = _run_dmesg_segfault
        assert log_checker.dmesg_check() is False
        run_calls = [
            call("dmesg | grep -i ' segfault '", return_output=True),
            call("echo '\nINFO found  segfault   Saving it\n'>> dmesg.log"),
            call("dmesg >> dmesg.log"),
            call("dmesg | grep -i 'Call Trace:'", return_output=True),
        ]
        # print(run_func.call_args_list)
        run_func.assert_has_calls(run_calls)
        run_func.reset_mock()

        run_func.side_effect = _run_dmesg_calltrace
        assert log_checker.dmesg_check() is False
        run_calls = [
            call("dmesg | grep -i ' segfault '", return_output=True),
            call("dmesg | grep -i 'Call Trace:'", return_output=True),
            call("echo '\nINFO found Call Trace:  Saving it\n'>> dmesg.log"),
            call("dmesg >> dmesg.log"),
        ]
        # print(run_func.call_args_list)
        run_func.assert_has_calls(run_calls)
        run_func.reset_mock()

    @patch("stqe.host.logchecker.kernel_check")
    def test_check_all(self, check_func):
        check_func.return_value = False
        assert log_checker.check_all() is False



# Copyright (C) 2022 Red Hat, Inc.
# This file is part of libsan.
#
# libsan is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# libsan is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with libsan.  If not, see <http://www.gnu.org/licenses/>.


import io
import os
import sys
import unittest

from stqe.host import tc


class TestTC(unittest.TestCase):
    def test_init(self):
        test_obj = tc.TestClass()
        log_msg = "my test"
        test_obj.tpass(log_msg)
        test_obj.tfail(log_msg)
        test_obj.tskip(log_msg)
        # initializing it again should preserve the values
        test_obj = tc.TestClass()
        assert len(test_obj.tc_pass) == 1
        assert len(test_obj.tc_fail) == 1
        assert len(test_obj.tc_skip) == 1

        # initializing after tend should clear the values
        test_obj.tend()
        test_obj = tc.TestClass()
        assert len(test_obj.tc_pass) == 0
        assert len(test_obj.tc_fail) == 0
        assert len(test_obj.tc_skip) == 0

    def test_tlog(self):
        test_obj = tc.TestClass()

        log_msg = "my test"
        new_callable = io.StringIO()
        sys.stdout = new_callable
        test_obj.tlog(log_msg)
        sys.stdout = sys.__stdout__
        assert new_callable.getvalue() == f"{log_msg}\n"

    def test_tpass(self):
        test_obj = tc.TestClass()

        log_msg = "my test"
        new_callable = io.StringIO()
        sys.stdout = new_callable
        assert os.path.exists(test_obj.test_log) is False
        test_obj.tpass(log_msg)
        sys.stdout = sys.__stdout__
        assert new_callable.getvalue() == "{}{}\n".format(test_obj.tc_sup_status["pass"], log_msg)
        assert log_msg in open(test_obj.test_log).read()  # noqaSIM115
        os.remove(test_obj.test_log)

    def test_tfail(self):
        test_obj = tc.TestClass()

        log_msg = "my test"
        new_callable = io.StringIO()
        sys.stdout = new_callable
        assert os.path.exists(test_obj.test_log) is False
        test_obj.tfail(log_msg)
        sys.stdout = sys.__stdout__
        assert new_callable.getvalue() == "{}{}\n".format(test_obj.tc_sup_status["fail"], log_msg)
        assert log_msg in open(test_obj.test_log).read()  # noqaSIM115
        os.remove(test_obj.test_log)

    def test_tskip(self):
        test_obj = tc.TestClass()

        log_msg = "my test"
        new_callable = io.StringIO()
        sys.stdout = new_callable
        assert os.path.exists(test_obj.test_log) is False
        test_obj.tskip(log_msg)
        sys.stdout = sys.__stdout__
        assert new_callable.getvalue() == "{}{}\n".format(test_obj.tc_sup_status["skip"], log_msg)
        assert log_msg in open(test_obj.test_log).read()  # noqaSIM115
        os.remove(test_obj.test_log)

    def test_tend_pass(self):
        test_obj = tc.TestClass()
        log_msg = "my test"
        test_obj.tskip(log_msg)
        test_obj.tpass(log_msg)
        assert test_obj.tend() is True

    def test_tend_fail(self):
        test_obj = tc.TestClass()
        log_msg = "my test"
        test_obj.tskip(log_msg)
        test_obj.tfail(log_msg)
        test_obj.tpass(log_msg)
        assert test_obj.tend() is False

    def test_log_submit(self):
        test_obj = tc.TestClass()

        test_obj.log_submit(test_obj.test_log)



#!/usr/bin/python


import os

from libsan.host.cmdline import run
from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def delete_backstore():
    errors = []

    arguments = {
        "wwn": None,
        "lun": None,
        "ip_address": None,
        "ip_port": None,
        "tag": None,
    }

    fileio_file = None
    path = os.environ["fmf_backstore"]
    if path == "fileio":
        fileio_file = read_var("FILEIO_FILE")

    for arg in arguments:
        try:
            arguments[arg] = os.environ["fmf_" + arg]
        except KeyError:
            pass

    arguments["name"] = read_var(path.upper() + "_NAME")

    target = TargetCLI(path="/backstores/%s" % path)

    atomic_run(
        "Deleting bacstore %s" % arguments["name"],
        command=target.delete,
        errors=errors,
        **target.remove_nones(arguments),
    )

    if fileio_file and os.path.isfile(fileio_file):
        atomic_run("Removing file ", cmd="rm -f %s" % fileio_file, command=run, errors=errors)

    return errors


if __name__ == "__main__":
    errs = delete_backstore()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def fs_resize_success():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    old_fs_size = read_var("LSM_FS_SIZE")
    new_fs_size = environ["fmf_new_fs_size"]

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Resizing FS {} to size {} with protocol {}.".format(fs_id, new_fs_size, config["protocol"]),
            command=lsm.fs_resize,
            fs=fs_id,
            size=new_fs_size,
            force=True,
            errors=errors,
        )

        # FIXME: Check the size is changed.

        atomic_run(
            "Resizing FS {} back to size {} with protocol {}.".format(fs_id, old_fs_size, config["protocol"]),
            command=lsm.fs_resize,
            fs=fs_id,
            size=old_fs_size,
            force=True,
            errors=errors,
        )

    return errors


def fs_resize_fail():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail resizing without any paramethers with protocol %s" % config["protocol"],
                "command": lsm.fs_resize,
            },
            {
                "message": "Trying to fail resizing FS with wrong 'fs' 'wrong' with protocol %s" % config["protocol"],
                "command": lsm.fs_resize,
                "fs": "wrong",
            },
            {
                "message": "Trying to fail resizing FS with 'fs' name instead of ID with protocol %s"
                % config["protocol"],
                "command": lsm.fs_resize,
                "fs": fs_name,
            },
            {
                "message": "Trying to fail resizing FS without 'size' with protocol %s" % config["protocol"],
                "command": lsm.fs_resize,
                "fs": fs_id,
            },
            {
                "message": "Trying to fail resizing FS without 'fs' with protocol %s" % config["protocol"],
                "command": lsm.fs_resize,
                "size": 10,
            },
            {
                "message": "Trying to fail resizing FS with 'size' '-1' with protocol %s" % config["protocol"],
                "command": lsm.fs_resize,
                "fs": fs_id,
                "size": -1,
            },
            {
                "message": "Trying to fail resizing FS with 'size' '0' with protocol %s" % config["protocol"],
                "command": lsm.fs_resize,
                "fs": fs_id,
                "size": 0,
            },
            {
                "message": "Trying to fail resizing FS with 'size' 'B' with protocol %s" % config["protocol"],
                "command": lsm.fs_resize,
                "fs": fs_id,
                "size": "B",
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, force=True, **argument)
    return errors


def fs_resize_fail_not_enough_space():
    errors = []

    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail resizing FS with 'size' '8000P' with protocol %s" % config["protocol"],
                "command": lsm.fs_resize,
                "fs": fs_id,
                "size": "8000P",
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=4, errors=errors, force=True, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = fs_resize_success()
    if int(environ["fmf_tier"]) == 2:
        errs = fs_resize_fail()
        errs += fs_resize_fail_not_enough_space()
    exit(parse_ret(errs))



#!/usr/bin/python
# Copyright (C) 2017 Red Hat, Inc.
# iscsi_params.py is a part of python-stqe
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Martin Hoyer   <mhoyer@redhat.com>


import argparse
import random
import sys

from libsan.host import fio, iscsi, linux, lio, mp, scsi

from stqe.host.logchecker import check_all

t_iqn = "iqn.2017-11.com.redhat:params-test"  # Target IQN
parameters = [
    "HeaderDigest",
    "MaxRecvDataSegmentLength",
    "MaxXmitDataSegmentLength",
    "FirstBurstLength",
    "MaxBurstLength",
    "ImmediateData",
    "InitialR2T",
]  # Parameters to be changed.

# Same parameters in /etc/iscsid.conf
iscsid_params = [
    "node.conn[0].iscsi.HeaderDigest",
    "node.conn[0].iscsi.MaxRecvDataSegmentLength",
    "node.conn[0].iscsi.MaxXmitDataSegmentLength",
    "node.session.iscsi.FirstBurstLength",
    "node.session.iscsi.MaxBurstLength",
    "node.session.iscsi.ImmediateData",
    "node.session.iscsi.InitialR2T",
]


def generate_values():  # Generating random values from the pools for both target and initiator.
    tgt_values = []
    in_values = []

    # Pools for random choice.
    digest = [
        "None",
        "None,CRC32C",
        "CRC32C,None",
    ]  # Separate 'CRC32C' option to prevent 'CRC32C'+'None' situations
    byte = [
        "512",
        "1024",
        "2048",
        "4096",
        "8192",
        "16384",
        "32768",
        "65536",
        "131072",
        "262144",
        "524288",
        "1048576",
        "16777212",
    ]
    yesno = ["Yes", "No"]
    crc = random.getrandbits(2)  # Generates 0-3
    if linux.dist_ver() > 7:
        if crc == 0:
            tgt_values.append("CRC32C")
            in_values.append("CRC32C")
        else:
            tgt_values.append(random.choice(digest))
            in_values.append(random.choice(digest))

    for _ in range(4):
        tgt_values.append(random.choice(byte))
        in_values.append(random.choice(byte))

    for _ in range(2):
        tgt_values.append(random.choice(yesno))
        in_values.append(random.choice(yesno))

    return tgt_values, in_values


def discovery_login():  # Discovering and logging into targets on localhost. There should be only one.
    iscsi.discovery_st("127.0.0.1", disc_db=True, ifaces="default")

    if not iscsi.node_login(portal="127.0.0.1", udev_wait_time=5):
        print("FAIL: Unable log into the target")
        return False

    return True


def do_io(
    device,
):  # fio - running randwrite until the 256M device is full, then verifying written data using CRC32C.
    if not fio.install_fio():
        print("FAIL: Unable to install fio")
        return False

    fio_params = {
        "bs": "4k",
        "direct": 1,
        "ioengine": "libaio",
        "iodepth": 16,
        "verify": "crc32c",
        "verify_fatal": 1,
        "do_verify": 1,
        "rw": "randwrite",
        "runtime": 120,
    }  # adding runtime cap of 2 minutes

    if not fio.fio_stress(device, verbose=True, **fio_params):
        print("FIO I/O failed")
        return False

    return True


def cleanup():
    if not (iscsi.clean_up() and lio.lio_clearconfig()):
        print("FAIL: Unable to clean-up iSCSI environment")
        return False

    return True


def main():
    print("INFO: Checking logs before iscsi_params test")
    if not check_all():
        print("WARN: Detected an error on logchecker")

    parser = argparse.ArgumentParser()
    # noinspection PyTypeChecker
    parser.add_argument(
        "--iterations",
        "-i",
        default=10,
        required=False,
        dest="iterations",
        help="number of iterations",
        type=int,
    )

    args = parser.parse_args()

    if not iscsi.install():
        return False

    if linux.dist_ver() < 8:
        parameters.remove("HeaderDigest")
        iscsid_params.remove("node.conn[0].iscsi.HeaderDigest")

    for _ in range(args.iterations):
        target_val, initiator_val = generate_values()
        tgt_params_values = dict(list(zip(parameters, target_val)))
        in_params_values = dict(list(zip(iscsid_params, initiator_val)))
        iscsi.clean_up()

        # Installing targetcli, clearing previous configuration, creating 256M local LIO target.
        if not lio.lio_setup_iscsi_target(lun_size="256M", tgt_iqn=t_iqn):
            return False

        # Setting parameter values with targetcli.
        for param in tgt_params_values:
            if not lio.lio_iscsi_target_set_parameter(
                tgt_iqn=t_iqn,
                tpg="tpg1",
                group="parameter",
                attr_name=param,
                attr_value=tgt_params_values[param],
            ):
                return False

        if not iscsi.set_iscsid_parameter(in_params_values):  # Setting parameter values in /etc/iscsid.conf.
            return False

        if not discovery_login():
            return False

        # How to check params manually:
        # iscsiadm -m session -P2 | grep HeaderDigest | cut -d " " -f 2
        # cat /sys/class/iscsi_connection/connection*/header_digest
        # cat /sys/class/iscsi_session/session*/first_burst_len

        # Printing negotiated values.
        sid = iscsi.get_all_session_ids()
        if not sid:
            print("FAIL: Could not find session ID to use...")
            return False
        negotiated = iscsi.query_iscsi_session(sid[0])

        session_info_dict = {
            "HeaderDigest": "header_digest",
            "MaxRecvDataSegmentLength": "max_recv",
            "MaxXmitDataSegmentLength": "max_xmit",
            "FirstBurstLength": "first_burst",
            "MaxBurstLength": "max_burst",
            "ImmediateData": "immediate_data",
            "InitialR2T": "initial_r2t",
        }

        count = 0
        for p in parameters:
            t_value = target_val[count]
            i_value = initiator_val[count]
            n_value = negotiated[session_info_dict[p]]
            n_expected = None
            print(p + ":")
            print("Target: %-11s| Initiator: %-11s| Negotiated: %-11s" % (t_value, i_value, n_value))
            print("______________________________________________________________\n")

            # Check if negotiated values are correct
            if p == "HeaderDigest":
                n_expected = "CRC32C" if i_value.split(",")[0] == "CRC32C" else "None"

                if (i_value == "CRC32C,None") and (t_value == "None"):
                    n_expected = "None"

            elif p == "MaxRecvDataSegmentLength":
                n_expected = i_value

            elif (p == "MaxXmitDataSegmentLength") or (p == "MaxBurstLength") or (p == "FirstBurstLength"):
                n_expected = t_value if int(t_value) < int(i_value) else i_value

            elif p == "ImmediateData":
                n_expected = "Yes" if t_value == "Yes" and i_value == "Yes" else "No"

            elif p == "InitialR2T":
                n_expected = "No" if t_value == "No" and i_value == "No" else "Yes"

            if p == "FirstBurstLength":
                # FirstBurstLength cannot be higher than MaxBurstLength
                if int(target_val[count + 1]) < int(initiator_val[count + 1]):
                    exp_max_burst = target_val[count + 1]
                else:
                    exp_max_burst = initiator_val[count + 1]

                if int(n_expected) > int(exp_max_burst):
                    n_expected = exp_max_burst

            if n_expected != n_value:
                print(f"FAIL: expected: {n_expected}, negotiated: {n_value}")
                return False

            count += 1

        # Choosing device to run IO on.
        if linux.is_service_running(mp.mp_service_name()):
            mpaths = mp.get_free_mpaths()
            if mpaths:
                test_dev = "/dev/mapper/" + list(mpaths.keys())[0]
            else:
                print("FAIL: multipathd is running but no free mpath devices found.")
                with open("/etc/multipath.conf") as file:
                    print("/etc/multipath.conf:\n\n", file.read())
                return False
        else:
            dev = scsi.get_free_disks(filter_only={"model": "params-test_lun"})
            if not dev:
                print("FAIL: Could not find device to use...")
                return False
            test_dev = "/dev/" + list(dev.keys())[0]

        if not do_io(test_dev):
            return False

        if not check_all():
            print("FAIL: Detected error on logchecker")
            return False

    return True


if main() is False:
    cleanup()
    sys.exit(1)

if not cleanup():
    sys.exit(1)



#!/usr/bin/python


import sys

from libsan.host.cmdline import run
from libsan.host.dmpd import (
    cache_check,
    cache_dump,
    cache_repair,
    cache_restore,
    get_help,
    get_version,
)
from libsan.host.linux import install_package
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.lvm import lv_activate, lv_convert, lv_create, vg_create, vg_remove

import stqe.host.tc
from stqe.host.atomic_run import atomic_run

TC = None


def init(args):
    print("INFO: Initializing test case")
    errors = []

    atomic_run(
        "Creating loopdev 1 - 'fast' device",
        name=args["loop1"],
        size=args["loop1_size"],
        command=create_loopdev,
        errors=errors,
    )

    atomic_run(
        "Creating loopdev 2 - 'slow' device",
        name=args["loop2"],
        size=args["loop2_size"],
        command=create_loopdev,
        errors=errors,
    )

    atomic_run(
        "Creating VG",
        vg_name=args["group"],
        pv_name="/dev/" + args["loop1"] + " /dev/" + args["loop2"],
        command=vg_create,
        errors=errors,
    )

    atomic_run(
        "Creating cache metadata volume",
        vg_name=args["group"] + " /dev/" + args["loop1"],
        lv_name=args["meta"],
        options=["-L 12"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Creating origin volume",
        vg_name=args["group"] + " /dev/" + args["loop2"],
        lv_name=args["origin"],
        options=["-L 2G"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Creating cache data volume",
        vg_name=args["group"] + " /dev/" + args["loop1"],
        lv_name=args["data"],
        options=["-L 1G"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Creating cache pool",
        vg_name=args["group"],
        lv_name=args["data"],
        options=[
            "-y --type cache-pool",
            "--cachemode writeback",
            "--poolmetadata {}/{}".format(args["group"], args["meta"]),
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Creating cache logical volume",
        vg_name=args["group"],
        lv_name=args["origin"],
        options=[
            "-y",
            "--type cache",
            "--cachepool {}/{}".format(args["group"], args["data"]),
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Creating filesystem on cache logical volume",
        cmd="mkfs.ext4 /dev/{}/{}".format(args["group"], args["origin"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Splitting cache logical volume",
        vg_name=args["group"],
        lv_name=args["origin"],
        options=["-y", "--splitcache"],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Creating swap LV",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=["-L 100"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Swapping metadata",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=[
            "-y",
            "--cachepool " + args["group"] + "/" + args["data"],
            "--poolmetadata ",
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Activating swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_activate,
        errors=errors,
    )

    if len(errors) == 0:
        TC.tpass("Initialization passed")
    else:
        TC.tfail("Initialization failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def clean(args):
    print("INFO: Cleaning up")
    errors = []

    atomic_run(
        "Removing VG",
        vg_name=args["group"],
        force=True,
        command=vg_remove,
        errors=errors,
    )

    atomic_run(
        "Deleting loopdev loop1",
        name=args["loop1"],
        command=delete_loopdev,
        errors=errors,
    )

    atomic_run(
        "Deleting loopdev loop2",
        name=args["loop2"],
        command=delete_loopdev,
        errors=errors,
    )

    atomic_run(
        "Deleting metadata file",
        cmd="rm -f /var/tmp/metadata",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Deleting repair metadata file",
        cmd="rm -f /var/tmp/metadata_repair",
        command=run,
        errors=errors,
    )

    if len(errors) == 0:
        TC.tpass("Cleanup passed")
    else:
        TC.tfail("Cleanup failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors]))
        print(errors)
        return 1
    return 0


def install_packages(packages):
    for package in packages:
        install_package(package)


def runtime_test(args):
    print("\n#######################################\n")
    print(
        "INFO: Testing cache tools runtime provided by device_mapper_persistent_data, iteration No. %s"
        % args["iteration"],
    )

    errors = []

    atomic_run(
        "Checking metadata",
        source_lv=args["swap"],
        source_vg=args["group"],
        command=cache_check,
        errors=errors,
    )

    atomic_run(
        "Checking metadata with clear-need-check-flag",
        source_lv=args["swap"],
        source_vg=args["group"],
        clear_needs_check_flag=True,
        command=cache_check,
        errors=errors,
    )

    atomic_run(
        "Checking metadata with super-block-only",
        source_lv=args["swap"],
        source_vg=args["group"],
        super_block_only=True,
        command=cache_check,
        errors=errors,
    )

    atomic_run(
        "Checking metadata with few paramethers",
        source_vg=args["group"],
        source_lv=args["swap"],
        skip_discards=True,
        skip_mappings=True,
        skip_hints=True,
        command=cache_check,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata to standard output",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=cache_dump,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for cache of 64 blocks and 128 size",
        cmd="cache_metadata_size --block-size 64 --device-size 128",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for cache of 128 nr blocks",
        cmd="cache_metadata_size --nr-blocks 128 --max-hint-width 4",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata to file",
        source_vg=args["group"],
        source_lv=args["swap"],
        repair=True,
        output="/var/tmp/metadata",
        command=cache_dump,
        errors=errors,
    )

    atomic_run(
        "Checking metadata file",
        False,
        source_file="/var/tmp/metadata",
        command=cache_check,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with options",
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        quiet=True,
        override_metadata_version=1,
        metadata_version=1,
        command=cache_restore,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata from file",
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        command=cache_restore,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata to file",
        target_file="/var/tmp/metadata_repair",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=cache_repair,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata from file",
        source_file="/var/tmp/metadata_repair",
        target_vg=args["group"],
        target_lv=args["swap"],
        command=cache_repair,
        errors=errors,
    )

    atomic_run(
        "Simulating TTY for cache_restore",
        cmd="script --return -c 'cache_restore -i /var/tmp/metadata -o /dev/mapper/%s-%s' /dev/null"
        % (args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking metadata",
        source_vg=args["group"],
        source_lv=args["swap"],
        quiet=True,
        command=cache_check,
        errors=errors,
    )

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing cache tools of device_mapper_persistent_data passed, iteration No. %s" % args["iteration"])
    else:
        TC.tfail(
            "Testing cache tools of device_mapper_persistent_data failed with following errors: \n\t'"
            + "\n\t ".join([str(i) for i in errors])
            + "\nIteration No. %s" % args["iteration"],
        )
        return 1
    return 0


def errors_test(args):
    print("\n#######################################\n")
    print(
        "INFO: Testing cache tools errors provided by device_mapper_persistent_data, iteration No. %s"
        % args["iteration"],
    )

    errors = []

    functions = [
        "cache_check",
        "cache_dump",
        "cache_metadata_size",
        "cache_repair",
        "cache_restore",
    ]

    # Sanity to check for missing input
    for func in functions:
        atomic_run("Validating missing input", False, cmd=func, command=run, errors=errors)

    # Sanity to check with wrong input
    for func in functions:
        atomic_run(
            "Validating wrong input",
            False,
            cmd=func + " wrong",
            command=run,
            errors=errors,
        )

    # Sanity to check with wrong option
    for func in functions:
        atomic_run(
            "Validating wrong option",
            False,
            cmd=func + " -wrong",
            command=run,
            errors=errors,
        )

    # Sanity to check with wrong -- option
    for func in functions:
        atomic_run(
            "Validating wrong -- option",
            False,
            cmd=func + " --wrong",
            command=run,
            errors=errors,
        )

    # Sanity to check present functions with -h
    for func in functions:
        atomic_run("Checking help of command", cmd="%s" % func, command=get_help, errors=errors)

    # Sanity to check present functions with -V
    for func in functions:
        atomic_run(
            "Checking version of command",
            cmd="%s" % func,
            command=get_version,
            errors=errors,
        )

    atomic_run(
        "Checking metadata of non-metadata file",
        False,
        cmd="cache_check README",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking metadata of non-existent file",
        False,
        cmd="cache_check WRONG",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking metadata of non-regular file",
        False,
        cmd="cache_check /dev/mapper/control",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for cache of 64 blocks",
        False,
        cmd="cache_metadata_size --block-size 64",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for cache of 128 size",
        False,
        cmd="cache_metadata_size --device-size 128",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for cache of 64 blocks and 128 size and 128 nr blocks",
        False,
        cmd="cache_metadata_size --block-size 64 --device-size 128 --nr-blocks 128",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata without output",
        False,
        cmd="cache_repair -i /var/tmp/metadata_repair",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with wrong options",
        False,
        cmd="cache_restore -i /var/tmp/metadata -o /dev/mapper/{}-{} --wrong test".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with wrong metadata version",
        False,
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        metadata_version=12445,
        command=cache_restore,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with wrong source",
        False,
        cmd="cache_restore -i /var/tmp/wrong.wrong -o /dev/mapper/{}-{}".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with bit source",
        False,
        source_file="/var/tmp/metadata_repair",
        target_vg=args["group"],
        target_lv=args["swap"],
        command=cache_restore,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata without output",
        False,
        cmd="cache_restore -i /var/tmp/metadata",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with options",
        pkg=["device-mapper-persistent-data", "0.7.3-2"],
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        omit_clean_shutdown=True,
        command=cache_restore,
        errors=errors,
    )

    atomic_run(
        "Checking metadata",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=cache_check,
        errors=errors,
    )

    # FIXME: Using BZ1255552 for this, find other way
    atomic_run(
        "Corrupting mappings on metadata device",
        False,
        source_file="Makefile",
        target_vg=args["group"],
        target_lv=args["swap"],
        command=cache_restore,
        errors=errors,
    )

    atomic_run(
        "Checking corrupted mappings",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        command=cache_check,
        errors=errors,
    )

    atomic_run(
        "Trying to fail while dumping metadata",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        output="/var/tmp/metadata",
        command=cache_dump,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata",
        source_vg=args["group"],
        source_lv=args["swap"],
        target_file="/var/tmp/metadata_repair",
        command=cache_repair,
        errors=errors,
    )

    atomic_run(
        "Corrupting metadata on device",
        cmd="echo 'nothing' >> /dev/mapper/{}-{}".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Trying to fail while repairing metadata",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        target_file="/var/tmp/metadata_repair",
        command=cache_repair,
        errors=errors,
    )

    atomic_run(
        "Trying to fail while dumping metadata",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        output="/var/tmp/metadata",
        command=cache_dump,
        errors=errors,
    )

    atomic_run(
        "Checking corrupted metadata",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        command=cache_check,
        errors=errors,
    )

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass(
            "Testing cache tools errors of device_mapper_persistent_data passed, iteration No. %s" % args["iteration"],
        )
    else:
        TC.tfail(
            "Testing cache tools errors of device_mapper_persistent_data failed with following errors: \n\t'"
            + "\n\t ".join([str(i) for i in errors])
            + "\nIteration No. %s" % args["iteration"],
        )
        return 1
    return 0


def main():
    # Initialize Test Case
    global TC
    TC = stqe.host.tc.TestClass()

    args = {
        "loop1": "loop1",
        "loop1_size": 2048,
        "loop2": "loop2",
        "loop2_size": 4128,
        "group": "vgtest",
        "origin": "origin",
        "data": "cache_data",
        "meta": "cache_meta",
        "pool": "cache_pool",
        "swap": "swapvol",
    }

    # Initialization
    packages = ["device-mapper-persistent-data"]
    install_packages(packages)

    init(args)

    # run it 3 times
    for i in range(1):
        args["iteration"] = i
        runtime_test(args)
        errors_test(args)

    clean(args)

    if not TC.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/python


from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret


def fs_limit():
    errors = []
    stratis = Stratis()
    dev = create_loopdev(size=10240)
    pool_name = "fs_limit"

    atomic_run(
        "Creating pool",
        pool_name=pool_name,
        blockdevs=dev,
        command=stratis.pool_create,
        errors=errors,
    )

    print("INFO: Creating 100 filesystems with 512MiB size.")
    for fs in range(0, 100):
        atomic_run(
            f"Creating filesystem: fs{fs}",
            pool_name=pool_name,
            fs_name=f"fs{fs}",
            fs_size="512MiB",
            command=stratis.fs_create,
            errors=errors,
        )

    atomic_run(
        "Creating fs number 101, this should fail because limit is set to 100 by default",
        expected_out=[
            "ERROR: The pool limit of 100 filesystems has already been reached; increase"
            " the filesystem limit on the pool to continue",
        ],
        expected_ret=1,
        pool_name=pool_name,
        fs_name="fs100",
        fs_size="512MiB",
        command=stratis.fs_create,
        errors=errors,
    )

    atomic_run(
        "Setting fs limit to 101",
        pool_name=pool_name,
        fs_amount=101,
        command=stratis.pool_set_fs_limit,
        errors=errors,
    )

    atomic_run(
        "Creating fs number 101, this should pass because limit has been set to 101",
        pool_name=pool_name,
        fs_name="fs100",
        fs_size="512MiB",
        command=stratis.fs_create,
        errors=errors,
    )

    atomic_run(
        "Creating fs number 102, this should fail because limit is set to 101",
        expected_out=[
            "ERROR: The pool limit of 101 filesystems has already been reached; increase"
            " the filesystem limit on the pool to continue",
        ],
        expected_ret=1,
        pool_name=pool_name,
        fs_name="fs101",
        fs_size="512MiB",
        command=stratis.fs_create,
        errors=errors,
    )

    for fs in range(0, 101):
        atomic_run(
            f"Destroying filesystem: fs{fs}",
            pool_name=pool_name,
            fs_name=f"fs{fs}",
            command=stratis.fs_destroy,
            errors=errors,
        )

    atomic_run(
        f"Destroying pool: {pool_name}",
        pool_name=pool_name,
        command=stratis.pool_destroy,
        errors=errors,
    )

    atomic_run("Removing loopdev", name=dev, command=delete_loopdev, errors=errors)

    return errors


if __name__ == "__main__":
    errs = fs_limit()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_dependants_rm_success():
    errors = []

    src_vol = read_var("LSM_VOL_ID")
    dst_vol = read_var("LSM_VOL_ID_2")
    rep_type = "CLONE"

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Replicating volume %s to volume %s with type %s to create dependency with protocol %s."
            % (src_vol, dst_vol, rep_type, config["protocol"]),
            command=lsm.volume_replicate_range,
            src_vol=src_vol,
            dst_vol=dst_vol,
            rep_type=rep_type,
            src_start=0,
            dst_start=0,
            count=100,
            force=True,
            errors=errors,
        )

        for vol in [src_vol, dst_vol]:
            _, data = atomic_run(
                "Checking for dependants of volume %s." % vol,
                command=lsm.volume_dependants,
                vol=vol,
                return_output=True,
                errors=errors,
            )
            if data != "False" and vol == dst_vol:
                msg = "FAIL: Volume %s should have no dependants." % vol
                print(msg)
                errors.append(msg)

            elif data != "True" and vol == src_vol:
                msg = "FAIL: Volume %s should have dependants." % vol
                print(msg)
                errors.append(msg)

        atomic_run(
            f"Removing dependency between replicated volumes {src_vol} and {dst_vol}",
            command=lsm.volume_dependants_rm,
            vol=src_vol,
            errors=errors,
        )

        for vol in [src_vol, dst_vol]:
            _, data = atomic_run(
                "Checking for dependants of volume %s." % vol,
                command=lsm.volume_dependants,
                vol=vol,
                return_output=True,
                errors=errors,
            )
            if data != "False":
                msg = "FAIL: Volume %s should have no dependants." % vol
                print(msg)
                errors.append(msg)

    return errors


def volume_dependants_rm_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail getting dependants of volume without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_dependants_rm,
            },
            {
                "message": "Trying to fail replicating volume range with WRONG volume with protocol %s"
                % config["protocol"],
                "vol": "WRONG",
                "command": lsm.volume_dependants_rm,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = volume_dependants_rm_success()
    if int(environ["fmf_tier"]) == 2:
        errs = volume_dependants_rm_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_cache_info_success():
    errors = []

    vol = read_var("LSM_VOL_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Getting RAM cache info for volume {} with protocol {}.".format(vol, config["protocol"]),
            command=lsm.volume_cache_info,
            vol=vol,
            errors=errors,
        )
    return errors


def volume_cache_info_fail():
    errors = []

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail getting RAM cache info without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_cache_info,
            },
            {
                "message": "Trying to fail getting RAM cache info with WRONG volume with protocol %s"
                % config["protocol"],
                "vol": "WRONG",
                "command": lsm.volume_cache_info,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def volume_cache_info_offline():
    errors = []
    # TODO: Check if one can get RAM cache info for disabled volume, write test for it
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = volume_cache_info_success()
    if int(environ["fmf_tier"]) == 2:
        errs = volume_cache_info_fail()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var, write_var


def create_fs():
    print("INFO: Creating snapshot of FS.")
    errors = []

    snap_name = environ["fmf_fs_snap_name"]
    fs_id = read_var("LSM_FS_ID")

    lsm = LibStorageMgmt(disable_check=True, **list(yield_lsm_config())[0])

    _, data = atomic_run(
        "Creating snapshot of FS '%s'" % fs_id,
        command=lsm.fs_snap_create,
        name=snap_name,
        fs=fs_id,
        return_output=True,
        errors=errors,
    )

    for line in data.splitlines():
        if snap_name in line:
            fs_snap_id = line.split()[0].strip()
            atomic_run(
                "Writing var LSM_FS_ID",
                command=write_var,
                var={"LSM_FS_SNAP_ID": fs_snap_id},
                errors=errors,
            )

    return errors


if __name__ == "__main__":
    errs = create_fs()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.fio import fio_stress
from libsan.host.linux import mkdir, mount, rmdir, umount
from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def basic_stress():
    errors = []

    stratis = Stratis()
    pool_name = "pool_stress"
    fs_name = "fs_stress"
    mnt_dir = "basic_stress"
    stratis_fs_path = f"/dev/stratis/{pool_name}/{fs_name}"
    blockdev = read_var("STRATIS_DEVICE")

    atomic_run(
        "Creating pool",
        pool_name=pool_name,
        blockdevs=blockdev,
        command=stratis.pool_create,
        errors=errors,
    )

    atomic_run(
        "Creating FS",
        pool_name=pool_name,
        fs_name=fs_name,
        command=stratis.fs_create,
        errors=errors,
    )

    atomic_run(f"Creating dir: {mnt_dir}", new_dir=mnt_dir, command=mkdir, errors=errors)

    atomic_run(
        f"Mounting filesystem: {fs_name}",
        device=stratis_fs_path,
        mountpoint=mnt_dir,
        command=mount,
        errors=errors,
    )

    arguments = [
        {
            "message": "Checking stratis status to get info on data.",
            "command": stratis.fs_list,
        },
        # FIXME: Get max size of device to run stress test
        {
            "message": "Starting fio stress on stratis fs",
            "command": fio_stress,
            "directory": f"{mnt_dir}",
            "of": None,
            "verbose": True,
            # fio settings
            "ioengine": environ["fmf_ioengine"],
            "direct": environ["fmf_direct"],
            "group_reporting": environ["fmf_group_reporting"],
            "bs": environ["fmf_bs"],
            "iodepth": environ["fmf_iodepth"],
            "numjobs": environ["fmf_numjobs"],
            "rw": environ["fmf_rw"],
            "rwmixread": environ["fmf_rwmixread"],
            "randrepeat": environ["fmf_randrepeat"],
            "norandommap": environ["fmf_norandommap"],
            "size": environ["fmf_size"],
            "runtime": int(environ["fmf_runtime"]) * 60,
        },
        {
            "message": "Checking stratis status to get info on data.",
            "command": stratis.fs_list,
        },
    ]

    for argument in arguments:
        atomic_run(errors=errors, **argument)

    atomic_run("Unmounting filesystem", mountpoint=mnt_dir, command=umount, errors=errors)

    atomic_run("Removing mnt dir", dir_name=mnt_dir, command=rmdir, errors=errors)

    atomic_run(
        f"Destroying filesystem: fs{fs_name}",
        pool_name=pool_name,
        fs_name=fs_name,
        command=stratis.fs_destroy,
        errors=errors,
    )

    atomic_run(
        f"Destroying pool: {pool_name}",
        pool_name=pool_name,
        command=stratis.pool_destroy,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = basic_stress()
    exit(parse_ret(errs))



"""stratis.py: Module with test specific method for Stratis."""

# Copyright (C) 2019 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
from stqe.host.fmf_tools import get_env_args
from stqe.host.persistent_vars import read_var


def get_stratis_args(stratis_object):
    """Returns dict of arguments passed from FMF env vars
    :return: dict.
    """
    arguments = dict(stratis_object.arguments)
    # Stratis does not use kwargs, only positional ones. SO we need to add them here manually.
    positional_args = [
        "pool_name",
        "blockdevs",
        "current",
        "new",
        "fs_name",
        "origin_name",
        "snapshot_name",
        "new_name",
        "key_desc",
        "keyfile_path",
        "cmd",
        "clevis",
        "thumbprint",
        "tang_url",
        "trust_url",
        "binding_method",
        "pool_error_code",
        "pool_uuid",
        "debug_subcommand",
        "fs_size",
        "fs_amount",
        "pool_overprovision",
        "no_overprovision",
        "fs_amount",
        "unlock_method",
        "stopped_pools",
    ]
    return get_env_args(list(arguments) + list(positional_args), read_var("FILE_NAMES"), {})



#!/usr/bin/python


from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.fmf_tools import get_env_args, get_func_from_string
from stqe.host.persistent_vars import read_env
from stqe.host.stratis import get_stratis_args


def call_func(args, stratis_object, errors):  # noqaARG001
    name = read_env("fmf_name")
    if "init_cache/success" in name or "add_cache/success" in name or "add_data/success" in name:
        if "single" in name:
            # Need to take only 1 device
            args["blockdevs"] = args["blockdevs"].pop()
        if "add_cache/success/multiple" in name or "init_cache/success/multiple" in name:
            # using all would trigger error, cache is limited to 32TB
            args["blockdevs"] = args["blockdevs"][:2]
    if (
        "fail/device_instead_of_pool_name" in name
        or "blockdev/list/fail/device_instead_of_name" in name
        or "fs/list/fail/device_instead_of_name" in name
    ):
        args["pool_name"] = args["pool_name"].pop()
    if "snapshot/fail/device_instead_of_snapshot_name" in name:
        args["origin_name"] = args["origin_name"].pop()
    if "pool/create/fail/no_pool_name" in name:
        args["blockdevs"] = args["blockdevs"].pop()

    return args


def stratis_cli():
    errors = []
    stratis = Stratis()
    args = get_stratis_args(stratis_object=stratis)
    # some extra args for some setups
    args.update(get_env_args({}))
    args = call_func(args, stratis, errors)
    if args is None:
        return errors

    print("stqe/stratis_cli args is %s" % args)
    command_string = args["command"]
    args["command"] = get_func_from_string(stratis, args.pop("command"), local_functions=globals())

    ret = atomic_run(errors=errors, **args)

    check_clean(
        ret=ret,
        stratis_object=stratis,
        args=args,
        errors=errors,
        command=command_string,
    )

    return errors


def check_clean(**kwargs):
    if "pool/rename/success" in read_env("fmf_name"):
        _, data = atomic_run(
            "Listing stratis pools.",
            command=kwargs["stratis_object"].pool_list,
            return_output=True,
            errors=kwargs["errors"],
        )
        if kwargs["args"]["new"] not in data:
            msg = "FAIL: Did not rename stratis pool {} to {}.".format(
                kwargs["args"]["current"],
                kwargs["args"]["new"],
            )
            print(msg)
            kwargs["errors"].append(msg)

    if "fs/rename/success" in read_env("fmf_name"):
        _, data = atomic_run(
            "Listing stratis fs on pool.",
            command=kwargs["stratis_object"].fs_list,
            pool_name=kwargs["args"]["pool_name"],
            return_output=True,
            errors=kwargs["errors"],
        )
        if kwargs["args"]["new_name"] not in data:
            msg = "FAIL: Did not rename stratis pool {} to {}.".format(
                kwargs["args"]["fs_name"],
                kwargs["args"]["new_name"],
            )
            print(msg)
            kwargs["errors"].append(msg)

    if "key/" in read_env("fmf_name"):
        _, data = atomic_run(
            "Listing stratis pools.",
            command=kwargs["stratis_object"].pool_list,
            return_output=True,
            errors=kwargs["errors"],
        )
        _, data = atomic_run(
            "Listing blockdev .",
            command=kwargs["stratis_object"].blockdev_list,
            return_output=True,
            errors=kwargs["errors"],
        )

        _, data = atomic_run(
            "Listing blockdev .",
            command=kwargs["stratis_object"].key_list,
            return_output=True,
            errors=kwargs["errors"],
        )

    return


if __name__ == "__main__":
    errs = stratis_cli()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run
from libsan.host.linux import service_start, service_stop

from stqe.host.atomic_run import atomic_run, parse_ret


def stratis_pid():
    errors = []

    atomic_run(
        "Stopping stratisd service",
        service_name="stratisd",
        command=service_stop,
        errors=errors,
    )

    atomic_run(
        "Removing symbolic link /var/run",
        cmd="unlink /var/run",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Starting stratisd service",
        service_name="stratisd",
        command=service_start,
        errors=errors,
    )

    atomic_run(
        "Restoring symbolic link to /run",
        cmd="ln -s /run /var/run",
        command=run,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = stratis_pid()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def fs_dependants_rm_success():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    fs_cloned_name = read_var("LSM_FS_CLONED_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            f"Cloning FS {fs_id} to FS {fs_cloned_name}.",
            command=lsm.fs_clone,
            src_fs=fs_id,
            dst_name=fs_cloned_name,
            return_output=True,
            errors=errors,
        )
        fs_cloned_id = [line.split()[0].strip() for line in data.splitlines() if fs_cloned_name in line][0]

        atomic_run(
            "Removing dependants of FS {} with protocol {}.".format(fs_id, config["protocol"]),
            command=lsm.fs_dependants_rm,
            fs=fs_id,
            errors=errors,
        )

        # this should return 'False', because we just removed the dependant
        _, data = atomic_run(
            "Checking dependants of FS {} with protocol {}.".format(fs_id, config["protocol"]),
            command=lsm.fs_dependants,
            fs=fs_id,
            return_output=True,
            errors=errors,
        )
        if data != "False":
            errors.append(
                "Removing dependants of FS {} did not return 'False', but {}."
                " Dependants did not get removed.".format(fs_id, data),
            )

        # FIXME: Add removing dependants for file

        atomic_run(
            "Removing cloned FS {} with protocol {}.".format(fs_cloned_name, config["protocol"]),
            command=lsm.fs_delete,
            fs=fs_cloned_id,
            force=True,
            errors=errors,
        )

    return errors


def fs_dependants_rm_fail():
    errors = []

    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail removing dependants without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.fs_dependants_rm,
            },
            {
                "message": "Trying to fail removing dependants FS with wrong 'fs' 'wrong' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_dependants_rm,
                "fs": "wrong",
            },
            {
                "message": "Trying to fail removing dependants FS with 'fs' name instead of ID with protocol %s"
                % config["protocol"],
                "command": lsm.fs_dependants_rm,
                "fs": fs_name,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def fs_dependants_rm_fail_no_state_change():
    errors = []

    fs_id = read_var("LSM_FS_ID")
    fs_cloned_name = read_var("LSM_FS_CLONED_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Removing dependants of FS {} with protocol {} to hit NO_STATE_CHANGE.".format(fs_id, config["protocol"]),
            command=lsm.fs_dependants_rm,
            fs=fs_id,
            expected_ret=4,
            errors=errors,
        )

        _, data = atomic_run(
            f"Cloning FS {fs_id} to FS {fs_cloned_name}.",
            command=lsm.fs_clone,
            src_fs=fs_id,
            dst_name=fs_cloned_name,
            return_output=True,
            errors=errors,
        )
        fs_cloned_id = [line.split()[0].strip() for line in data.splitlines() if fs_cloned_name in line][0]

        atomic_run(
            "Removing dependants of FS {} with protocol {}.".format(fs_id, config["protocol"]),
            command=lsm.fs_dependants_rm,
            fs=fs_id,
            errors=errors,
        )

        atomic_run(
            "Removing cloned FS {} with protocol {}.".format(fs_cloned_name, config["protocol"]),
            command=lsm.fs_delete,
            fs=fs_cloned_id,
            force=True,
            errors=errors,
        )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = fs_dependants_rm_success()
    if int(environ["fmf_tier"]) == 2:
        errs = fs_dependants_rm_fail()
        errs += fs_dependants_rm_fail_no_state_change()
    exit(parse_ret(errs))



#!/usr/bin/python


from libsan.host.cmdline import run
from libsan.host.mp import get_disks_of_mpath

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var, write_var


def setup_multipath():
    disk = read_var("VDO_DEVICE")
    print("INFO: Setting up dm-multipath on device %s." % disk)
    errors = []

    _, mpaths = atomic_run(
        message="Getting multipaths device",
        command=run,
        return_output=True,
        cmd="multipath -l -v 1",
        errors=errors,
    )
    mpaths = [mpath.strip() for mpath in mpaths.splitlines()]
    disk = disk.split("/").pop()

    mpath_disk = disk
    for mpath in mpaths:
        possible_disks = get_disks_of_mpath(mpath)
        if disk in possible_disks:
            mpath_disk = "mapper/%s" % mpath

    atomic_run(
        "Writing var VDO_DEVICE",
        command=write_var,
        var={"VDO_DEVICE": "/dev/%s" % mpath_disk},
        errors=errors,
    )

    atomic_run(message="Listing block devices.", command=run, cmd="lsblk", errors=errors)

    return errors


if __name__ == "__main__":
    errs = setup_multipath()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def iscsi_chap_success():
    errors = []

    init = read_var("LSM_INIT_ID_CHAP")
    in_user = read_var("LSM_CHAP_IN_USER")
    in_pass = read_var("LSM_CHAP_IN_PASS")
    out_user = read_var("LSM_CHAP_OUT_USER")
    out_pass = read_var("LSM_CHAP_OUT_PASS")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Setting up CHAP auth without credentials with protocol %s" % config["protocol"],
            command=lsm.iscsi_chap,
            init=init,
            errors=errors,
        )

        atomic_run(
            "Setting up CHAP auth with IN credentials with protocol %s" % config["protocol"],
            command=lsm.iscsi_chap,
            init=init,
            in_user=in_user,
            in_pass=in_pass,
            errors=errors,
        )

        atomic_run(
            "Setting up CHAP auth with OUT credentials with protocol %s" % config["protocol"],
            command=lsm.iscsi_chap,
            init=init,
            out_user=out_user,
            out_pass=out_pass,
            errors=errors,
        )

        atomic_run(
            "Setting up CHAP auth with all credentials with protocol %s" % config["protocol"],
            command=lsm.iscsi_chap,
            init=init,
            in_user=in_user,
            in_pass=in_pass,
            out_user=out_user,
            out_pass=out_pass,
            errors=errors,
        )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = iscsi_chap_success()
    exit(parse_ret(errs))



#!/usr/bin/python


from stqe.host.atomic_run import parse_ret
from stqe.host.persistent_vars import clean_var, read_var, write_var


def setup_manual():
    # this just checks if the device is created.
    data = read_var("VDO_DEVICE")

    # remove newline
    if data.endswith("\n"):
        data = data.rstrip("\n")
        clean_var("VDO_DEVICE")
        write_var({"VDO_DEVICE": data})

    print("INFO: Will run tests on device: '%s'" % data)
    return []


if __name__ == "__main__":
    errs = setup_manual()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.cmdline import run
from libsan.host.linux import is_module_loaded, load_module, unload_module

from stqe.host.atomic_run import atomic_run, parse_ret


def uds():
    # Tries to load/unload uds kernel module.
    errors = []
    arguments = []

    # clean up modules first, to be sure (kvdo blocks uds)
    for module in ["kvdo", "uds"]:
        _, data = run(cmd="lsmod | grep %s" % module, return_output=True)
        if data != "":
            arguments += [
                {
                    "message": "Unloading module '%s'." % module,
                    "command": unload_module,
                    "module_name": module,
                },
            ]

    arguments += [
        {"message": "Loading module 'uds'.", "command": load_module, "module": "uds"},
        {
            "message": "Checking if 'uds' is loaded.",
            "command": is_module_loaded,
            "module_name": "uds",
        },
        {
            "message": "Unloading module 'uds'.",
            "command": unload_module,
            "module_name": "uds",
        },
    ]

    for argument in arguments:
        atomic_run(errors=errors, **argument)

    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = uds()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def volume_unmask_fail():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail unmasking access group to volume without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.volume_unmask,
            },
            {
                "message": "Trying to fail unmasking access group to volume without volume with protocol %s"
                % config["protocol"],
                "ag": ag_id,
                "command": lsm.volume_unmask,
            },
            {
                "message": "Trying to fail unmasking access group to volume without access group with protocol %s"
                % config["protocol"],
                "vol": vol_id,
                "command": lsm.volume_unmask,
            },
            {
                "message": "Trying to fail unmasking access group to volume with wrong volume with protocol %s"
                % config["protocol"],
                "ag": ag_id,
                "vol": "WRONG",
                "command": lsm.volume_unmask,
            },
            {
                "message": "Trying to fail unmasking access group to volume with wrong access group with protocol %s"
                % config["protocol"],
                "vol": vol_id,
                "ag": "WRONG",
                "command": lsm.volume_unmask,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def volume_unmask_no_state_change():
    errors = []

    vol_id = read_var("LSM_VOL_ID")
    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Removing access for access group %s to volume %s to hit NO_STATE_CHANGE with protocol %s."
                % (ag_id, vol_id, config["protocol"]),
                "vol": vol_id,
                "ag": ag_id,
                "command": lsm.volume_unmask,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=4, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 2:
        errs = volume_unmask_fail()
        errs += volume_unmask_no_state_change()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.cmdline import run
from libsan.host.iscsi import discovery_st, node_login
from libsan.host.linux import hostname, install_package, is_service_running
from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.iscsi import auto_conf
from stqe.host.nfs_lock import setup_nfs_lock_iscsi
from stqe.host.persistent_vars import write_var


def setup_iscsi(test_type="stratis-general"):
    print("INFO: Setting up iSCSI with LUN %s." % test_type)
    errors = []

    install_package("iscsi-initiator-utils")

    if "brq" in hostname():
        auto_conf(test_type=test_type)

    else:
        wwn = f"iqn.1994-05.com.redhat:{test_type}"
        target = TargetCLI(path="/backstores/fileio")
        atomic_run(
            "Creating fileio backstore",
            command=target.create,
            name="stratis-test",
            file_or_dev="/var/tmp/stratis-test.img",
            size="10G",
            errors=errors,
        )

        target.path = "/iscsi"
        atomic_run("Creating iscsi target", wwn=wwn, command=target.create, errors=errors)

        target.path = "/iscsi/" + wwn + "/" + "tpg1" + "/" + "luns"

        atomic_run(
            "Creating lun with fileio backstore",
            command=target.create,
            storage_object="/backstores/fileio/stratis-test",
            errors=errors,
        )

        target.path = "/iscsi/" + wwn + "/" + "tpg1"

        atomic_run(
            "Setting parameter generate_node_acls to '1'",
            generate_node_acls=1,
            group="attribute",
            command=target.set,
            errors=errors,
        )

        atomic_run(
            "Setting parameter demo_mode_write_protect to '0'",
            demo_mode_write_protect=0,
            group="attribute",
            command=target.set,
            errors=errors,
        )

        atomic_run(
            "Discovering local target",
            target="127.0.0.1",
            disc_db=True,
            command=discovery_st,
            errors=errors,
        )

        atomic_run(
            "Trying to login to portal: 127.0.0.1",
            portal="127.0.0.1",
            command=node_login,
            errors=errors,
        )

    # on RHEL 7 there is no LUN name in /dev/disk/by-path, just ID.
    _, local_disk = atomic_run(
        message="Getting iSCSI disk",
        command=run,
        return_output=True,
        cmd="ls -la /dev/disk/by-path/ | grep '%s'" % test_type.replace("_", "-"),
        errors=errors,
    )
    if not local_disk:
        _, local_disk = atomic_run(
            message="Getting iSCSI disk through iscsiadm",
            command=run,
            return_output=True,
            cmd="iscsiadm -m session -P 3 | grep disk",
            errors=errors,
        )
        if local_disk:
            # remove fail from 'ls -la ...'
            errors.pop()
        disk = local_disk.split()[3]
    else:
        disk = local_disk.split("../../").pop()

    if is_service_running("multipathd"):
        ret, multipath = atomic_run(
            message="Getting mpath device",
            command=run,
            return_output=True,
            cmd="multipath -l /dev/%s -v 1" % disk,
            errors=errors,
        )

        atomic_run(
            message="Flushing mpath device %s" % multipath,
            command=run,
            cmd="multipath -f %s" % multipath,
            errors=errors,
        )

        atomic_run(
            message="Removing mpath device from disk %s" % disk,
            command=run,
            cmd="multipath -W /dev/%s" % disk,
            errors=errors,
        )

        atomic_run(
            message="Triggering udev",
            command=run,
            cmd="udevadm trigger; udevadm settle",
            errors=errors,
        )

    if "brq" in hostname():
        ret = atomic_run(
            "NFS locking iscsi disk %s." % disk,
            command=setup_nfs_lock_iscsi,
            disk_name=disk,
            errors=errors,
        )
        if ret:
            print("FAIL: Could not get NFS lock for iscsi LUN.")
            return errors

    atomic_run(
        "Cleaning superblock of device /dev/%s" % disk,
        command=run,
        cmd="dd if=/dev/zero of=/dev/%s bs=1M count=10" % disk,
        errors=errors,
    )

    atomic_run(
        "Writing var STRATIS_DEVICE",
        command=write_var,
        var={"STRATIS_DEVICE": "/dev/%s" % disk},
        errors=errors,
    )

    atomic_run("Listing block devices.", command=run, cmd="lsblk", errors=errors)

    return errors


if __name__ == "__main__":
    lun_name = "stratis-general"
    try:
        lun_name = environ["fmf_lun_name"]
    except KeyError:
        pass
    errs = setup_iscsi(lun_name)
    exit(parse_ret(errs))



#!/usr/bin/python -u
# -u is for unbuffered stdout
# Copyright (C) 2016 Red Hat, Inc.
# python-stqe is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# python-stqe is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with python-stqe.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Bruno Goncalves   <bgoncalv@redhat.com>


import argparse
import sys

from libsan.host import dm, linux, lvm
from libsan.host.cmdline import run
from libsan.host.loopdev import create_loopdev, delete_loopdev

import stqe.host.tc

TestObj = None

loop_dev = {}

vg_name = "vgcache"
slow_lv = "slow_lv"
fast_lv = "cache_lv"
fast_meta_lv = "cache_metadata_lv"
mnt_point = "/mnt/cache_test"


def start_test(filesystem):
    global TestObj

    print(80 * "#")
    print("INFO: Starting test on FS %s" % filesystem)
    print(80 * "#")

    _clean_up()

    global loop_dev
    loop_dev["slow_dev"] = create_loopdev(size=4096)
    if not loop_dev["slow_dev"]:
        # Skip test if we are not able to create loopdev
        TestObj.tskip("Could not create loop device to be used as slow_dev")
        return 2

    loop_dev["fast_dev"] = create_loopdev(size=2048)
    if not loop_dev["fast_dev"]:
        TestObj.tskip("Could not create loop device to be used as fast_dev")
        return 2

    if not lvm.vg_create(vg_name, "{} {}".format(loop_dev["slow_dev"], loop_dev["fast_dev"])):
        TestObj.tfail('Could not create VG "%s"' % vg_name)
        return 1

    print("INFO: Creating Slow LV")
    if run("lvcreate -n {} -L 3G {} {}".format(slow_lv, vg_name, loop_dev["slow_dev"])) != 0:
        TestObj.tfail("Could not create %s" % slow_lv)
        return 1
    print("INFO: Creating Cache LV")
    if run("lvcreate -n {} -L 1G {} {}".format(fast_lv, vg_name, loop_dev["fast_dev"])) != 0:
        TestObj.tfail("Could not create %s" % fast_lv)
        return 1
    print("INFO: Creating  Cache metadata LV")
    if run("lvcreate -n {} -L 12M {} {}".format(fast_meta_lv, vg_name, loop_dev["fast_dev"])) != 0:
        TestObj.tfail("Could not create %s" % fast_meta_lv)
        return 1

    print("INFO: Creating cache pool logical volume")
    if (
        run(
            "lvconvert --yes --type cache-pool --cachemode writethrough --poolmetadata %s/%s %s/%s"
            % (vg_name, fast_meta_lv, vg_name, fast_lv),
        )
        != 0
    ):
        TestObj.tfail("Could not create cache pool LV")
        return 1

    print("INFO: Creating cache logical volume")
    if run(f"lvconvert --yes --type cache --cachepool {vg_name}/{fast_lv} {vg_name}/{slow_lv}") != 0:
        TestObj.tfail("Could not create cache LV")
        return 1

    print("INFO: Displaying LVs")
    run("lvs -a -o +devices")

    dm.dm_show_table()

    lv_device = f"/dev/mapper/{vg_name}-{slow_lv}"
    print(f"INFO:Going to create FS({filesystem}) on {lv_device}")
    if run(f"mkfs.{filesystem} {lv_device}") != 0:
        TestObj.tfail(f"Could not create FS({filesystem}) on {lv_device}")
        return 1

    if not linux.mkdir(mnt_point):
        TestObj.tfail(f"Could not create directory {mnt_point} - FS({filesystem})")
        return 1

    if not linux.mount(lv_device, mnt_point):
        TestObj.tfail(f"Could not mount {mnt_point} - FS({filesystem})")
        return 1

    print("INFO: Going create file on %s" % mnt_point)
    filename = "file1.img"
    if run(f"dd conv=fdatasync if=/dev/urandom of={mnt_point}/{filename} bs=1M count=2000") != 0:
        TestObj.tfail(f"Could not create file on {mnt_point} - FS({filesystem})")
        return 1

    linux.sync()
    run("lvs -a")

    print(80 * "#")
    TestObj.tpass("PASS: Test on FS %s" % filesystem)
    print(80 * "#")

    return 0


def _clean_up():
    global vg_name, loop_dev

    linux.umount(mnt_point)

    # make sure any failed device is removed
    run("dmsetup remove_all")
    if not lvm.vg_remove(vg_name, force=True):
        TestObj.tfail('Could not delete VG "%s"' % vg_name)

    if loop_dev:
        for dev in loop_dev:
            if loop_dev[dev]:
                if not lvm.pv_remove(loop_dev[dev]):
                    TestObj.tfail('Could not delete PV "%s"' % loop_dev[dev])
                linux.sleep(1)
                if not delete_loopdev(loop_dev[dev]):
                    TestObj.tfail("Could not remove loop device %s" % loop_dev[dev])


def main():
    global TestObj

    parser = argparse.ArgumentParser(description="cache_basic")
    parser.add_argument(
        "--filesystem",
        "-f",
        required=False,
        dest="fs",
        help="Filesystem name",
        metavar="filesytem",
    )

    args = parser.parse_args()

    TestObj = stqe.host.tc.TestClass()

    linux.install_package("lvm2")

    filesystem = linux.get_default_fs()

    if args.fs:
        filesystem = args.fs

    ret = start_test(filesystem)

    _clean_up()

    if not TestObj.tend():
        print("FAIL: test failed")
        sys.exit(1)

    if ret == 2:
        print("SKIP: Test has been skipped because loop device could not be created!")
        sys.exit(2)

    print("PASS: Test pass")
    sys.exit(0)


main()



#!/usr/bin/python


import os

from libsan.host.lio import TargetCLI

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import read_var


def create_pscsi():
    errors = []

    target = TargetCLI(path="/backstores/pscsi")

    scsi_id = read_var("SCSI_ID")
    name = os.environ["fmf_pscsi_name"]

    atomic_run(
        "Creating backstore pscsi object with name: %s" % name,
        name=name,
        dev=scsi_id,
        command=target.create,
        errors=errors,
    )

    atomic_run(
        "Deleting pscsi object %s" % name,
        name=name,
        command=target.delete,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = create_pscsi()
    exit(parse_ret(errs))



#!/usr/bin/python


import sys

from libsan.host.cmdline import run
from libsan.host.dmpd import (
    get_help,
    get_version,
    thin_check,
    thin_delta,
    thin_dump,
    thin_ls,
    thin_repair,
    thin_restore,
    thin_rmap,
    thin_trim,
)
from libsan.host.linux import install_package
from libsan.host.loopdev import create_loopdev, delete_loopdev
from libsan.host.lvm import (
    lv_activate,
    lv_convert,
    lv_create,
    lv_deactivate,
    lv_remove,
    vg_create,
    vg_remove,
)

import stqe.host.tc
from stqe.host.atomic_run import atomic_run

TC = None


def _mount_thin_lv(vg_name, lv_name):
    ret = run("mkdir -p /mnt/%s" % lv_name, verbose=True)
    ret += run(f"mkfs.ext4 /dev/{vg_name}/{lv_name}", verbose=True)
    ret += run(f"mount /dev/{vg_name}/{lv_name} /mnt/{lv_name}/", verbose=True)
    return ret


def _unmount_thin_lv(lv_name):
    ret = run("umount -f /mnt/%s/" % lv_name, verbose=True)
    ret += run("rm -rf /mnt/%s" % lv_name, verbose=True)
    return ret


def _metadata_snapshot(vg_name, lv_name):
    ret = run(f"dmsetup suspend /dev/mapper/{vg_name}-{lv_name}-tpool", verbose=True)
    ret += run(
        f"dmsetup message /dev/mapper/{vg_name}-{lv_name}-tpool 0 reserve_metadata_snap",
        verbose=True,
    )
    ret += run(f"dmsetup resume /dev/mapper/{vg_name}-{lv_name}-tpool", verbose=True)
    return ret


def init(args):
    print("INFO: Initializing test case")
    errors = []

    atomic_run(
        "Creating loopdev",
        expected_ret="/dev/loop1",
        name=args["loop"],
        size=args["loop_size"],
        command=create_loopdev,
        errors=errors,
    )

    atomic_run(
        "Creating VG",
        vg_name=args["group"],
        pv_name="/dev/" + args["loop"],
        command=vg_create,
        errors=errors,
    )

    atomic_run(
        "Creating thin pool",
        vg_name=args["group"],
        lv_name=args["pool"],
        options=["-T", "-L 100"],
        command=lv_create,
        errors=errors,
    )

    # create few LVs to increase transaction ID and be able to do thin_delta
    for i in range(args["number of vols"]):
        atomic_run(
            "Creating thin LV No. %s" % i,
            vg_name=args["group"] + "/" + args["pool"],
            lv_name=args["vol"] + str(i),
            options=["-T", "-V 10"],
            command=lv_create,
            errors=errors,
        )

        atomic_run(
            "Mounting LV No. %s" % i,
            expected_ret=0,
            vg_name=args["group"],
            lv_name=args["vol"] + str(i),
            command=_mount_thin_lv,
            errors=errors,
        )

        atomic_run(
            "Unmounting LV No. %s" % i,
            expected_ret=0,
            lv_name=args["vol"] + str(i),
            command=_unmount_thin_lv,
            errors=errors,
        )

        atomic_run(
            "Deactivating thin LV No. %s" % i,
            lv_name=args["vol"] + str(i),
            vg_name=args["group"],
            command=lv_deactivate,
            errors=errors,
        )

    atomic_run(
        "Creating metadata snapshot",
        expected_ret=0,
        lv_name=args["pool"],
        vg_name=args["group"],
        command=_metadata_snapshot,
        errors=errors,
    )

    atomic_run(
        "Deactivating pool",
        lv_name=args["pool"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Creating swap LV",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=["-L 100"],
        command=lv_create,
        errors=errors,
    )

    atomic_run(
        "Deactivating swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Swapping metadata",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=[
            "-y",
            "--thinpool " + args["group"] + "/" + args["pool"],
            "--poolmetadata ",
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Activating swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_activate,
        errors=errors,
    )

    if len(errors) == 0:
        TC.tpass("Initialization passed")
    else:
        TC.tfail("Initialization failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors]))
        return 1
    return 0


def clean(args):
    print("INFO: Cleaning up")
    errors = []

    # restoring metadata device in case it is corrupted
    atomic_run(
        "Repairing metadata device",
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        quiet=True,
        command=thin_restore,
        errors=errors,
    )

    # thinpool got activated after checking its metadata to get bad checksum
    atomic_run(
        "Deactivating pool",
        lv_name=args["pool"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Deactivating swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Swapping back metadata",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=[
            "-y",
            "--thinpool " + args["group"] + "/" + args["pool"],
            "--poolmetadata ",
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Removing swap",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_remove,
        errors=errors,
    )

    atomic_run(
        "Removing thinpool",
        lv_name=args["pool"],
        vg_name=args["group"],
        command=lv_remove,
        errors=errors,
    )

    atomic_run(
        "Removing VG",
        vg_name=args["group"],
        force=True,
        command=vg_remove,
        errors=errors,
    )

    atomic_run("Deleting loopdev", name=args["loop"], command=delete_loopdev, errors=errors)

    atomic_run(
        "Deleting metadata file",
        cmd="rm -f /var/tmp/metadata",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Deleting repair metadata file",
        cmd="rm -f /var/tmp/metadata_repair",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Deleting snapshot metadata file",
        cmd="rm -f /var/tmp/metadata_snap",
        command=run,
        errors=errors,
    )

    if len(errors) == 0:
        TC.tpass("Cleanup passed")
    else:
        TC.tfail("Cleanup failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors]))
        print(errors)
        return 1
    return 0


def install_packages(packages):
    for package in packages:
        install_package(package)


def runtime_test(args):
    print("\n#######################################\n")
    print(
        "INFO: Testing thin tools runtime provided by device_mapper_persistent_data, iteration No. %s"
        % args["iteration"],
    )

    errors = []

    atomic_run(
        "Checking metadata",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=thin_check,
        errors=errors,
    )

    atomic_run(
        "Checking metadata with few paramethers",
        source_vg=args["group"],
        source_lv=args["swap"],
        super_block_only=True,
        skip_mappings=True,
        ignore_non_fatal_errors=True,
        command=thin_check,
        errors=errors,
    )

    atomic_run(
        "Listing information about thin LVs",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=thin_ls,
        errors=errors,
    )

    atomic_run(
        "Listing information about thin LVs without headers",
        source_vg=args["group"],
        source_lv=args["swap"],
        no_headers=True,
        command=thin_ls,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata to standard output without mappings",
        formatting="human_readable",
        source_vg=args["group"],
        source_lv=args["swap"],
        skip_mappings=True,
        command=thin_dump,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata to standard output",
        formatting="human_readable",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=thin_dump,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata to standard output from snapshot",
        formatting="human_readable",
        source_vg=args["group"],
        source_lv=args["swap"],
        snapshot=True,
        command=thin_dump,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata with dev-id",
        formatting="human_readable",
        source_vg=args["group"],
        source_lv=args["swap"],
        dev_id=args["number of vols"] - 1,
        command=thin_dump,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for pool of 64k blocks and 100M size",
        cmd="thin_metadata_size -b64k -s100m -m1 -um",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for pool of 64k blocks and 100M size",
        cmd="thin_metadata_size -b64k -s100m -m1 -um -n",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for pool of 64k blocks and 100M size",
        cmd="thin_metadata_size -b64k -s100m -m1 -um -nlong",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Calculating metadata size for pool of 64k blocks and 100M size",
        cmd="thin_metadata_size -b64k -s100m -m1 -um -nshort",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Outputting reverse map of metadata device",
        source_vg=args["group"],
        source_lv=args["swap"],
        region="0..-1",
        command=thin_rmap,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata to file",
        formatting="xml",
        source_vg=args["group"],
        source_lv=args["swap"],
        repair=True,
        output="/var/tmp/metadata",
        command=thin_dump,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata to file from snapshot",
        formatting="xml",
        source_vg=args["group"],
        source_lv=args["swap"],
        snapshot=True,
        output="/var/tmp/metadata_snap",
        command=thin_dump,
        errors=errors,
    )

    atomic_run(
        "Getting differences between thin LVs",
        source_vg=args["group"],
        source_lv=args["swap"],
        thin1=1,
        thin2=args["number of vols"] - 1,
        snapshot=True,
        command=thin_delta,
        errors=errors,
    )

    atomic_run(
        "Getting differences between thin LVs with --verbose",
        source_vg=args["group"],
        source_lv=args["swap"],
        thin1=1,
        thin2=args["number of vols"] - 1,
        verbosity=True,
        snapshot=True,
        command=thin_delta,
        errors=errors,
    )

    atomic_run(
        "Getting differences between the same LV",
        source_vg=args["group"],
        source_lv=args["swap"],
        thin1=1,
        thin2=1,
        snapshot=True,
        command=thin_delta,
        errors=errors,
    )

    atomic_run(
        "Getting differences between the same LV with --verbose",
        source_vg=args["group"],
        source_lv=args["swap"],
        thin1=1,
        thin2=1,
        verbosity=True,
        snapshot=True,
        command=thin_delta,
        errors=errors,
    )

    atomic_run(
        "Listing metadata output from snapshot",
        source_vg=args["group"],
        source_lv=args["swap"],
        snapshot=True,
        command=thin_ls,
        errors=errors,
    )

    # need to run everything on snapshot before this as thin_restore removes the metadata snapshot
    # Restoring metadata from snapshot is not supported, see BZ1477232
    atomic_run(
        "Restoring metadata to get 'couldn't zero superblock'",
        False,
        source_file="/var/tmp/metadata_snap",
        target_vg=args["group"],
        target_lv=args["swap"],
        command=thin_restore,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata",
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        command=thin_restore,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata from file to get 'Metadata is not large enough for superblock'",
        False,
        pkg=["device-mapper-persistent-data", "0.7.3-3"],
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        command=thin_repair,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata to file",
        target_file="/var/tmp/metadata_repair",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=thin_repair,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata from file",
        source_file="/var/tmp/metadata_repair",
        target_vg=args["group"],
        target_lv=args["swap"],
        command=thin_repair,
        errors=errors,
    )

    #   ##################### start of thin_trim #####################################
    # Need to swap back metadata to be able to activate thin pool
    atomic_run(
        "Swapping metadata",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=[
            "-y",
            "--thinpool " + args["group"] + "/" + args["pool"],
            "--poolmetadata ",
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Activating swap for thin_trim",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_activate,
        errors=errors,
    )

    atomic_run(
        "Activating pool for thin_trim",
        lv_name=args["pool"],
        vg_name=args["group"],
        command=lv_activate,
        errors=errors,
    )

    # thin_trim requires activated thin pool and its metadata, that was required beforehand
    atomic_run(
        "Discarding free space of pool",
        pkg=["device-mapper-persistent-data", "0.7.3-3"],
        data_vg=args["group"],
        data_lv=args["pool"],
        metadata_file="/var/tmp/metadata_repair",
        command=thin_trim,
        errors=errors,
    )

    atomic_run(
        "Deactivating pool after thin_trim",
        lv_name=args["pool"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Deactivating swap after thin_trim",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_deactivate,
        errors=errors,
    )

    atomic_run(
        "Swapping back metadata",
        vg_name=args["group"],
        lv_name=args["swap"],
        options=[
            "-y",
            "--thinpool " + args["group"] + "/" + args["pool"],
            "--poolmetadata ",
        ],
        command=lv_convert,
        errors=errors,
    )

    atomic_run(
        "Activating swap after thin_trim",
        lv_name=args["swap"],
        vg_name=args["group"],
        command=lv_activate,
        errors=errors,
    )
    #   ##################### end of thin_trim #####################################

    atomic_run(
        "Checking metadata",
        source_vg=args["group"],
        source_lv=args["swap"],
        command=thin_check,
        errors=errors,
    )

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass("Testing thin tools of device_mapper_persistent_data passed, iteration No. %s" % args["iteration"])
    else:
        TC.tfail(
            "Testing thin tools of device_mapper_persistent_data failed with following errors: \n\t'"
            + "\n\t ".join([str(i) for i in errors])
            + "\nIteration No. %s" % args["iteration"],
        )
        return 1
    return 0


def errors_test(args):
    print("\n#######################################\n")
    print(
        "INFO: Testing thin tools errors provided by device_mapper_persistent_data, iteration No. %s"
        % args["iteration"],
    )

    errors = []

    functions = [
        "thin_check",
        "thin_delta",
        "thin_dump",
        "thin_ls",
        "thin_metadata_size",
        "thin_repair",
        "thin_restore",
        "thin_rmap",
        "thin_trim",
    ]

    # Sanity to check for missing input
    for func in functions:
        atomic_run("Validating missing input", False, cmd=func, command=run, errors=errors)

    # Sanity to check with wrong input
    for func in functions:
        atomic_run(
            "Validating wrong input",
            False,
            cmd=func + " wrong",
            command=run,
            errors=errors,
        )

    # Sanity to check with wrong option
    for func in functions:
        atomic_run(
            "Validating wrong option",
            False,
            cmd=func + " -wrong",
            command=run,
            errors=errors,
        )

    # Sanity to check present functions with -h
    for func in functions:
        kwargs = {"pkg": ["device-mapper-persistent-data", "0.7.3-2"]} if func in ["thin_trim"] else {}
        atomic_run("Checking help of command", cmd=func, command=get_help, errors=errors, **kwargs)

    # Sanity to check present functions with -V
    for func in functions:
        if func in ["thin_trim", "thin_metadata_size"]:
            kwargs = {"pkg": ["device-mapper-persistent-data", "0.7.3-2"]}
        else:
            kwargs = {}
        atomic_run("Checking version of command", cmd=func, command=get_version, errors=errors, **kwargs)

    atomic_run(
        "Checking original pool metadata, should fail",
        False,
        source_vg=args["group"],
        source_lv=args["pool"],
        command=thin_check,
        errors=errors,
    )

    atomic_run(
        "Listing information about thin LVs",
        False,
        cmd='thin_ls /dev/mapper/{}-{} --format "WRONG"'.format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking thin_metadata_size inputs",
        False,
        cmd="thin_metadata_size -b 64",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking thin_metadata_size inputs",
        False,
        cmd="thin_metadata_size -b 64 -s 128",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking thin_metadata_size inputs",
        False,
        cmd="thin_metadata_size -b 25 -s 128 -m 10",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking thin_metadata_size inputs",
        False,
        cmd="thin_metadata_size -b 128 -s 64 -m 10",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking thin_metadata_size inputs",
        False,
        cmd="thin_metadata_size -u h",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking thin_metadata_size inputs",
        False,
        cmd="thin_metadata_size -n -n",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking thin_metadata_size inputs",
        False,
        cmd="thin_metadata_size -nlongshort",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Checking thin_metadata_size inputs",
        False,
        cmd="thin_metadata_size -b 128 -b 64",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata without output",
        False,
        cmd="thin_repair -i /var/tmp/metadata_repair",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata with wrong custom format",
        False,
        cmd="thin_dump /dev/mapper/{}-{} --format custom=wrong".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata with unknown format",
        False,
        cmd="thin_dump /dev/mapper/{}-{} --format wrong".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Dumping metadata with wrong dev-id",
        False,
        cmd="thin_dump /dev/mapper/{}-{} --dev-id wrong".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata to produce 'output file does not exist' error",
        False,
        cmd="thin_repair -i /dev/mapper/{}-{} -o /var/tmp/wrong.wrong".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Repairing metadata to produce 'output file too small' error",
        False,
        cmd="thin_repair -i /var/tmp/metadata -o /var/tmp/metadata",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Outputting reverse map of metadata device, should fail without region",
        False,
        pkg=["device-mapper-persistent-data", "0.7.3-2"],
        cmd="thin_rmap /dev/mapper/{}-{}".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Outputting reverse map of metadata device with bad region to get 'badly formed region (end <= begin)",
        False,
        cmd="thin_rmap /dev/mapper/{}-{} --region 0..0".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Outputting reverse map of metadata device with bad region to get "
        "'badly formed region (couldn't parse numbers)'",
        False,
        cmd="thin_rmap /dev/mapper/{}-{} --region 0...1".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Outputting reverse map of metadata device with bad region to get 'badly formed region (no dots)'",
        False,
        cmd="thin_rmap /dev/mapper/{}-{} --region 00".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Outputting reverse map of metadata device with wrong path to get 'Couldn't stat path'",
        False,
        cmd="thin_rmap --region 0..-1 /var/tmp/wrong.wrong",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Outputting reverse map of metadata device to get 'Metadata is not large enough for superblock'",
        False,
        cmd="thin_rmap --region 0..-1 /var/tmp/metadata",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Getting differences with thin1 ID out of range",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        thin1=-1,
        thin2=args["number of vols"] - 1,
        command=thin_delta,
        errors=errors,
    )

    atomic_run(
        "Getting differences with thin2 ID out of range",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        thin1=1,
        thin2=args["number of vols"] + 1,
        command=thin_delta,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata without output",
        False,
        cmd="thin_restore -i /var/tmp/metadata",
        command=run,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with wrong options",
        False,
        cmd="thin_restore -i /var/tmp/metadata -o /dev/mapper/{}-{} --wrong test".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Restoring metadata with wrong source",
        False,
        cmd="thin_restore -i /var/tmp/wrong.wrong -o /dev/mapper/{}-{}".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Getting differences without thin2",
        False,
        cmd="thin_delta --thin1 1 /dev/mapper/{}-{}".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Corrupting metadata on device",
        cmd="echo 'nothing' >> /dev/mapper/{}-{}".format(args["group"], args["swap"]),
        command=run,
        errors=errors,
    )

    atomic_run(
        "Trying to fail while repairing metadata",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        target_file="/var/tmp/metadata_repair",
        command=thin_repair,
        errors=errors,
    )

    atomic_run(
        "Trying to fail listing volumes",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        command=thin_ls,
        errors=errors,
    )

    atomic_run(
        "Trying to fail while checking metadata",
        False,
        source_vg=args["group"],
        source_lv=args["swap"],
        command=thin_check,
        errors=errors,
    )

    atomic_run(
        "Trying to fail while dumping metadata from snapshot",
        False,
        formatting="human_readable",
        source_vg=args["group"],
        source_lv=args["swap"],
        snapshot=True,
        command=thin_dump,
        errors=errors,
    )

    # restoring metadata device after corrupting it
    atomic_run(
        "Repairing metadata device",
        source_file="/var/tmp/metadata",
        target_vg=args["group"],
        target_lv=args["swap"],
        quiet=True,
        command=thin_restore,
        errors=errors,
    )

    print("\n#######################################\n")

    if len(errors) == 0:
        TC.tpass(
            "Testing thin tools errors of device_mapper_persistent_data passed, iteration No. %s" % args["iteration"],
        )
    else:
        TC.tfail(
            "Testing thin tools errors of device_mapper_persistent_data failed with following errors: \n\t'"
            + "\n\t ".join([str(i) for i in errors])
            + "\nIteration No. %s" % args["iteration"],
        )
        return 1
    return 0


def main():
    # Initialize Test Case
    global TC
    TC = stqe.host.tc.TestClass()

    args = {
        "loop": "loop1",
        "loop_size": 2048,
        "group": "vgtest",
        "pool": "thinpool",
        "vol": "thinvol",
        "number of vols": 10,
        "swap": "swapvol",
    }

    # Initialization
    packages = ["device-mapper-persistent-data"]
    install_packages(packages)

    init(args)

    # run it 3 times
    for i in range(1):
        args["iteration"] = i
        runtime_test(args)
        errors_test(args)

    clean(args)

    if not TC.tend():
        print("FAIL: test failed")
        sys.exit(1)

    print("PASS: Test pass")
    sys.exit(0)


if __name__ == "__main__":
    main()



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def fs_snap_create_success():
    errors = []

    snap_name = read_var("LSM_FS_SNAP_NAME")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Creating snapshot {} on FS {} with protocol {}.".format(snap_name, fs_id, config["protocol"]),
            command=lsm.fs_snap_create,
            name=snap_name,
            fs=fs_id,
            return_output=True,
            errors=errors,
        )
        snap_id = [line.split()[0].strip() for line in data.splitlines() if snap_name in line][0]
        atomic_run(
            "Removing snapshot {} from FS {} with protocol {}.".format(snap_id, fs_id, config["protocol"]),
            command=lsm.fs_snap_delete,
            snap=snap_id,
            fs=fs_id,
            force=True,
            errors=errors,
        )
    return errors


def fs_snap_create_fail():
    errors = []

    snap_name = read_var("LSM_FS_SNAP_NAME")
    fs_id = read_var("LSM_FS_ID")
    fs_name = read_var("LSM_FS_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail creating snapshot on FS without any paramethers with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_create,
            },
            {
                "message": "Trying to fail creating snapshot on FS without 'fs' with protocol %s" % config["protocol"],
                "command": lsm.fs_snap_create,
                "name": snap_name,
            },
            {
                "message": "Trying to fail creating snapshot on FS without 'name' with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_create,
                "fs": fs_id,
            },
            {
                "message": "Trying to fail creating snapshot on FS by using fs-name instead of fs-ID with protocol %s"
                % config["protocol"],
                "command": lsm.fs_snap_create,
                "fs": fs_name,
                "name": snap_name,
            },
            {
                "message": "Trying to fail creating snapshot on NONEXISTENT FS with protocol %s" % config["protocol"],
                "command": lsm.fs_snap_create,
                "fs": "NONEXISTENT",
                "name": snap_name,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


def fs_snap_create_fail_name_conflict():
    errors = []

    snap_name = read_var("LSM_FS_SNAP_NAME")
    fs_id = read_var("LSM_FS_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        _, data = atomic_run(
            "Creating snapshot {} on FS {} with protocol {}.".format(snap_name, fs_id, config["protocol"]),
            command=lsm.fs_snap_create,
            name=snap_name,
            fs=fs_id,
            return_output=True,
            errors=errors,
        )
        snap_id = [line.split()[0].strip() for line in data.splitlines() if snap_name in line][0]

        atomic_run(
            "Creating snapshot %s on FS %s again to hit name conflict with protocol %s."
            % (snap_name, fs_id, config["protocol"]),
            command=lsm.fs_snap_create,
            name=snap_name,
            fs=fs_id,
            expected_ret=4,
            errors=errors,
        )

        atomic_run(
            "Removing snapshot {} from FS {} with protocol {}.".format(snap_id, fs_id, config["protocol"]),
            command=lsm.fs_snap_delete,
            snap=snap_id,
            fs=fs_id,
            force=True,
            errors=errors,
        )
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = fs_snap_create_success()
    if int(environ["fmf_tier"]) == 2:
        errs = fs_snap_create_fail()
        errs += fs_snap_create_fail_name_conflict()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.lsm import LibStorageMgmt

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.lsm import yield_lsm_config
from stqe.host.persistent_vars import read_var


def access_group_volumes_success():
    errors = []

    ag_id = read_var("LSM_AG_ID")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        atomic_run(
            "Listing volumes with access from AG {} with protocol {}.".format(ag_id, config["protocol"]),
            command=lsm.access_group_volumes,
            ag=ag_id,
            errors=errors,
        )

    return errors


def access_group_volumes_fail():
    errors = []

    ag_name = read_var("LSM_AG_NAME")

    for config in yield_lsm_config():
        lsm = LibStorageMgmt(disable_check=True, **config)

        arguments = [
            {
                "message": "Trying to fail listing volumes with access from AG without any paramethers "
                "with protocol %s" % config["protocol"],
                "command": lsm.access_group_volumes,
            },
            {
                "message": "Trying to fail listing volumes with access from AG with AG NONEXISTENT with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_volumes,
                "ag": "NONEXISTENT",
            },
            {
                "message": "Trying to fail listing volumes with access from with AG name instead of ID with protocol %s"
                % config["protocol"],
                "command": lsm.access_group_volumes,
                "ag": ag_name,
            },
        ]
        for argument in arguments:
            atomic_run(expected_ret=2, errors=errors, **argument)
    return errors


if __name__ == "__main__":
    if int(environ["fmf_tier"]) == 1:
        errs = access_group_volumes_success()
    if int(environ["fmf_tier"]) == 2:
        errs = access_group_volumes_fail()
    exit(parse_ret(errs))



#!/usr/bin/env python


import argparse
import sys

import stqe.host.nfs_lock as lock


def wait_nfs_lock(mpath, lock_type):
    print(f"Requesting NFS lock ({lock_type}) for mpath: {mpath}")
    lock_obj = lock.setup_nfs_lock_for_mpath(mpath)
    if not lock_obj:
        print("FAIL: Could not setup NFS lock for mpath: %s" % mpath)
        return False
    if not lock_obj.request_lock(lock_type):
        print(f"FAIL: Could not request lock: {lock_type} using {mpath}")
        return False
    print("INFO: Waiting for lock...")
    if not lock_obj.get_lock():
        print("FAIL: Could not get NFS lock")
        return False
    print("INFO: lock sucessfully acquired")
    return True


def release_nfs_lock(mpath, lock_type):
    print(f"Releasing NFS lock ({lock_type}) from mpath: {mpath}")
    lock_obj = lock.setup_nfs_lock_for_mpath(mpath)
    if not lock_obj:
        print("FAIL: Could not setup NFS lock for mpath: %s" % mpath)
        return False
    if not lock_obj.release_lock(lock_type):
        print(f"FAIL: Could not release lock: {lock_type} using {mpath}")
        return False
    print("INFO: Lock successfully released")
    return True


def main():
    parser = argparse.ArgumentParser(description="Tool useful for stqe environment")
    subparsers = parser.add_subparsers(help="Valid commands", dest="command")

    parser_wait_nfs_lock = subparsers.add_parser("wait-nfs-lock")
    parser_wait_nfs_lock.add_argument(
        "--mpath",
        "-m",
        required=True,
        dest="mpath",
        metavar="mpath_name",
        help="Mpath Name.",
    )
    parser_wait_nfs_lock.add_argument(
        "--lock-type",
        "-l",
        required=True,
        dest="lock_type",
        metavar="lock_type",
        help="Lock Type.",
    )

    parser_release_nfs_lock = subparsers.add_parser("release-nfs-lock")
    parser_release_nfs_lock.add_argument(
        "--mpath",
        "-m",
        required=True,
        dest="mpath",
        metavar="mpath_name",
        help="Mpath Name.",
    )
    parser_release_nfs_lock.add_argument(
        "--lock-type",
        "-l",
        required=True,
        dest="lock_type",
        metavar="lock_type",
        help="Lock Type.",
    )

    args = parser.parse_args()
    if args.command == "wait-nfs-lock":
        if wait_nfs_lock(args.mpath, args.lock_type):
            sys.exit(0)
        sys.exit(1)

    if args.command == "release-nfs-lock":
        if release_nfs_lock(args.mpath, args.lock_type):
            sys.exit(0)
        sys.exit(1)

    print("FAIL: Unsupported command: %s" % args.command)
    sys.exit(1)


main()



from libsan.host.stratis import Stratis

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import clean_var, read_env, read_var


def generate_key():
    errors = []
    id = ""
    stratis = Stratis()

    try:
        id = "_" + str(read_env("fmf_key_id"))
    except KeyError:
        pass

    key_desc = read_var(f"STRATIS_KEY_DESC{id}")
    keyfile_name = read_var(f"STRATIS_KEYFILE{id}")

    atomic_run(
        f"Unsetting stratis key {key_desc}.",
        command=stratis.key_unset,
        key_desc=key_desc,
        errors=errors,
    )

    atomic_run(
        f"Cleaning var STRATIS_KEYFILE{id}",
        command=clean_var,
        var=f"STRATIS_KEYFILE{id}",
        errors=errors,
    )

    atomic_run(
        f"Cleaning var STRATIS_KEYFILE_PATH{id}",
        command=clean_var,
        var=f"STRATIS_KEYFILE_PATH{id}",
        errors=errors,
    )

    atomic_run(
        f"Cleaning var STRATIS_KEYFILE{id}",
        command=clean_var,
        var=f"{keyfile_name}",
        errors=errors,
    )

    atomic_run(
        f"Cleaning var STRATIS_KEY_DESC{id}",
        command=clean_var,
        var=f"STRATIS_KEY_DESC{id}",
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = generate_key()
    exit(parse_ret(errs))



#!/usr/bin/python


from os import environ

from libsan.host.qemu_img import get_qcow_supported_options, qemu_create

from stqe.host.atomic_run import atomic_run, parse_ret
from stqe.host.persistent_vars import write_var


def create_qemu_img():
    errors = []

    name = environ["fmf_filename"]
    size = environ["fmf_img_size"]
    supported_options = get_qcow_supported_options()
    args = {"fmt": None, "img_path": "/var/tmp"}
    for arg in args:
        try:
            args[arg] = environ["fmf_" + arg]
        except KeyError:
            pass
    for option in supported_options:
        try:
            args[option] = environ["fmf_" + option]
        except KeyError:
            pass

    atomic_run(
        "Creating qemu disk image %s" % name, filename=name, size=size, command=qemu_create, errors=errors, **args
    )

    atomic_run(
        "Writing var IMAGE_NAME",
        command=write_var,
        var={"IMAGE_NAME": name},
        errors=errors,
    )

    atomic_run(
        "Writing var IMAGE_PATH",
        command=write_var,
        var={"IMAGE_PATH": args["img_path"]},
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = create_qemu_img()
    exit(parse_ret(errs))



#!/usr/bin/python


import os
from time import sleep

from libsan.host import linux, targetd

from stqe.host.atomic_run import atomic_run, parse_ret


def targetd_ssl_test():
    errors = []

    arguments = {}

    arguments["password"] = os.environ["fmf_ssl_password"]
    arguments["key"] = os.environ["fmf_key"]
    arguments["cert"] = os.environ["fmf_cert"]

    atomic_run(
        "Starting targetd SSL test",
        enable=True,
        command=targetd.ssl_change,
        errors=errors,
    )

    atomic_run(
        "Adding password to the config",
        password=arguments["password"],
        command=targetd.set_password,
        errors=errors,
    )

    atomic_run(
        "Creating SSL key",
        name=arguments["key"],
        command=targetd.gen_ssl_key,
        errors=errors,
    )

    atomic_run(
        "Creating SSL certificate",
        name=arguments["cert"],
        key_name=arguments["key"],
        command=targetd.gen_ssl_cert,
        errors=errors,
    )

    atomic_run(
        "Targetd configured properly.... starting targetd service",
        service_name="targetd",
        command=linux.service_restart,
        errors=errors,
    )

    print("Waiting 10 seconds for the service to restart")
    sleep(10)

    atomic_run("Getting service status", command=targetd.targetd_status, errors=errors)

    atomic_run("Checking TLS status", command=targetd.tls_status, errors=errors)

    atomic_run(
        "Stopping targetd service",
        service_name="targetd",
        command=linux.service_stop,
        errors=errors,
    )

    atomic_run("Removing password from config", command=targetd.set_password, errors=errors)

    atomic_run(
        "Removing SSL cert: %s" % arguments["cert"],
        name=arguments["cert"] + ".pem",
        command=targetd.remove_file,
        errors=errors,
    )

    atomic_run(
        "Removing SSL key: %s" % arguments["key"],
        name=arguments["key"] + ".pem",
        command=targetd.remove_file,
        errors=errors,
    )

    return errors


if __name__ == "__main__":
    errs = targetd_ssl_test()
    exit(parse_ret(errs))



#!/usr/bin/python

# This is a reproducer for Bug 1456019
# [Pegas1.0 EA2] cache_dump crashes with SIGSEGV when passed small file as an argument

# !/usr/bin/python

import sys

from libsan.host.cmdline import run
from libsan.host.linux import install_package

import stqe.host.tc
from stqe.host.atomic_run import atomic_run
from stqe.host.tc import TestClass

global tc


def cache_dump_test(tc):
    print("\n#######################################\n")
    print("INFO: Testing cache_dump for SIGSEV.")

    errors = []

    arguments = [
        {
            "message": "Creating very small file to dump to.",
            "cmd": "fallocate -l 200 /var/tmp/small_file",
            "command": run,
        },
        {
            "message": "Dumping metadata from small file",
            "cmd": "cache_dump /var/tmp/small_file",
            "command": run,
        },
        {
            "message": "Removing small file",
            "cmd": "rm -f /var/tmp/small_file",
            "command": run,
        },
    ]

    for argument in arguments:
        atomic_run(errors=errors, **argument)

    print("\n#######################################\n")

    if len(errors) == 0:
        tc.tpass("Testing cache_dump for SIGSEV passed.")
    else:
        tc.tfail(
            "Testing cache_dump for SIGSEV failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors])
        )
        return 1
    return 0


def thin_dump_test(tc):
    print("\n#######################################\n")
    print("INFO: Testing thin_dump for SIGSEV.")

    errors = []

    arguments = [
        {
            "message": "Creating very small file to dump to.",
            "cmd": "fallocate -l 200 /var/tmp/small_file",
            "command": run,
        },
        {
            "message": "Dumping metadata from small file",
            "cmd": "thin_dump /var/tmp/small_file",
            "command": run,
        },
        {
            "message": "Removing small file",
            "cmd": "rm -f /var/tmp/small_file",
            "command": run,
        },
    ]

    for argument in arguments:
        atomic_run(errors=errors, **argument)

    print("\n#######################################\n")

    if len(errors) == 0:
        tc.tpass("Testing thin_dump for SIGSEV passed.")
    else:
        tc.tfail(
            "Testing thin_dump for SIGSEV failed with following errors: \n\t'" + "\n\t ".join([str(i) for i in errors])
        )
        return 1
    return 0


def main():
    # Initialize Test Case
    test_class: TestClass = stqe.host.tc.TestClass()

    install_package("device-mapper-persistent-data")

    cache_dump_test(tc=test_class)
    thin_dump_test(tc=test_class)

    if not test_class.tend():
        print("FAIL: Test failed")
        sys.exit(1)

    print("PASS: Test passed")
    sys.exit(0)


if __name__ == "__main__":
    main()
